"""PRU Controller Parameters.

These parameters are used in PRUController class to
interact with BSMP devices.
"""

from ..bsmp.constants import ConstPSBSMP as _const
from ..psctrl.psmodel import PSModelFactory as _PSModelFactory


class PRUCParmsFBP:
    """FBP-specific PRUC parameters."""

    FREQ_SCAN = 5.0  # [Hz]

    # PS model parms
    model = _PSModelFactory.create('FBP')
    CONST_PSBSMP = model.bsmp_constants
    Entities = model.entities

    groups = dict()
    # reserved variable groups (not to be used)
    groups[_const.G_ALL] = tuple(sorted(Entities.list_variables(0)))
    groups[_const.G_READONLY] = tuple(sorted(Entities.list_variables(1)))
    groups[_const.G_WRITE] = tuple(sorted(Entities.list_variables(2)))
    # new variable groups useful for PRUController.
    groups[_const.G_ALLRELEVANT] = (
        # --- common variables
        CONST_PSBSMP.V_PS_STATUS,
        CONST_PSBSMP.V_PS_SETPOINT,
        CONST_PSBSMP.V_PS_REFERENCE,
        CONST_PSBSMP.V_FIRMWARE_VERSION,
        CONST_PSBSMP.V_COUNTER_SET_SLOWREF,
        CONST_PSBSMP.V_COUNTER_SYNC_PULSE,
        CONST_PSBSMP.V_SIGGEN_ENABLE,
        CONST_PSBSMP.V_SIGGEN_TYPE,
        CONST_PSBSMP.V_SIGGEN_NUM_CYCLES,
        CONST_PSBSMP.V_SIGGEN_N,
        CONST_PSBSMP.V_SIGGEN_FREQ,
        CONST_PSBSMP.V_SIGGEN_AMPLITUDE,
        CONST_PSBSMP.V_SIGGEN_OFFSET,
        CONST_PSBSMP.V_SIGGEN_AUX_PARAM,
        CONST_PSBSMP.V_WFMREF_SELECTED,
        CONST_PSBSMP.V_WFMREF_SYNC_MODE,
        CONST_PSBSMP.V_WFMREF_FREQUENCY,
        CONST_PSBSMP.V_WFMREF_GAIN,
        CONST_PSBSMP.V_WFMREF_OFFSET,
        CONST_PSBSMP.V_WFMREF0_START,
        CONST_PSBSMP.V_WFMREF0_END,
        CONST_PSBSMP.V_WFMREF0_IDX,
        CONST_PSBSMP.V_WFMREF1_START,
        CONST_PSBSMP.V_WFMREF1_END,
        CONST_PSBSMP.V_WFMREF1_IDX,
        CONST_PSBSMP.V_SCOPE_FREQUENCY,
        CONST_PSBSMP.V_SCOPE_DURATION,
        CONST_PSBSMP.V_SCOPE_SRC_DATA,
        # --- FBP variables ---
        CONST_PSBSMP.V_PS_SOFT_INTERLOCKS,
        CONST_PSBSMP.V_PS_HARD_INTERLOCKS,
        CONST_PSBSMP.V_I_LOAD,
        CONST_PSBSMP.V_V_LOAD,
        CONST_PSBSMP.V_V_DCLINK,
        CONST_PSBSMP.V_TEMP_SWITCHES,
        CONST_PSBSMP.V_DUTY_CYCLE,)
    groups[_const.G_SCAN] = (
        # --- common variables
        CONST_PSBSMP.V_PS_STATUS,
        CONST_PSBSMP.V_PS_SETPOINT,
        CONST_PSBSMP.V_PS_REFERENCE,
        # CONST_PSBSMP.V_FIRMWARE_VERSION,
        CONST_PSBSMP.V_COUNTER_SET_SLOWREF,
        CONST_PSBSMP.V_COUNTER_SYNC_PULSE,
        CONST_PSBSMP.V_SIGGEN_ENABLE,
        CONST_PSBSMP.V_SIGGEN_TYPE,
        CONST_PSBSMP.V_SIGGEN_NUM_CYCLES,
        CONST_PSBSMP.V_SIGGEN_N,
        CONST_PSBSMP.V_SIGGEN_FREQ,
        CONST_PSBSMP.V_SIGGEN_AMPLITUDE,
        CONST_PSBSMP.V_SIGGEN_OFFSET,
        CONST_PSBSMP.V_SIGGEN_AUX_PARAM,
        CONST_PSBSMP.V_WFMREF_SELECTED,
        CONST_PSBSMP.V_WFMREF_SYNC_MODE,
        CONST_PSBSMP.V_WFMREF_FREQUENCY,
        CONST_PSBSMP.V_WFMREF_GAIN,
        CONST_PSBSMP.V_WFMREF_OFFSET,
        CONST_PSBSMP.V_WFMREF0_START,
        CONST_PSBSMP.V_WFMREF0_END,
        CONST_PSBSMP.V_WFMREF0_IDX,
        CONST_PSBSMP.V_WFMREF1_START,
        CONST_PSBSMP.V_WFMREF1_END,
        CONST_PSBSMP.V_WFMREF1_IDX,
        CONST_PSBSMP.V_SCOPE_FREQUENCY,
        CONST_PSBSMP.V_SCOPE_DURATION,
        CONST_PSBSMP.V_SCOPE_SRC_DATA,
        # --- FBP variables ---
        CONST_PSBSMP.V_PS_SOFT_INTERLOCKS,
        CONST_PSBSMP.V_PS_HARD_INTERLOCKS,
        CONST_PSBSMP.V_I_LOAD,
        CONST_PSBSMP.V_V_LOAD,
        CONST_PSBSMP.V_V_DCLINK,
        CONST_PSBSMP.V_TEMP_SWITCHES,)
    groups[_const.G_SOFB] = (
        CONST_PSBSMP.V_PS_SETPOINT1,
        CONST_PSBSMP.V_PS_SETPOINT2,
        CONST_PSBSMP.V_PS_SETPOINT3,
        CONST_PSBSMP.V_PS_SETPOINT4,
        CONST_PSBSMP.V_PS_REFERENCE1,
        CONST_PSBSMP.V_PS_REFERENCE2,
        CONST_PSBSMP.V_PS_REFERENCE3,
        CONST_PSBSMP.V_PS_REFERENCE4,
        CONST_PSBSMP.V_I_LOAD1,
        CONST_PSBSMP.V_I_LOAD2,
        CONST_PSBSMP.V_I_LOAD3,
        CONST_PSBSMP.V_I_LOAD4,)


class PRUCParmsFBP_DCLink:
    """FBP_DCLink-specific PRUC parameters."""

    FREQ_SCAN = 2.0  # [Hz]

    # PS model parms
    model = _PSModelFactory.create('FBP_DCLink')
    CONST_PSBSMP = model.bsmp_constants
    Entities = model.entities

    groups = dict()
    # reserved variable groups (not to be used)
    groups[_const.G_ALL] = tuple(sorted(Entities.list_variables(0)))
    groups[_const.G_READONLY] = tuple(sorted(Entities.list_variables(1)))
    groups[_const.G_WRITE] = tuple(sorted(Entities.list_variables(2)))
    # new variable groups useful for PRUController.
    groups[_const.G_ALLRELEVANT] = (
        # --- common variables
        CONST_PSBSMP.V_PS_STATUS,
        CONST_PSBSMP.V_PS_SETPOINT,
        CONST_PSBSMP.V_PS_REFERENCE,
        CONST_PSBSMP.V_FIRMWARE_VERSION,
        CONST_PSBSMP.V_COUNTER_SET_SLOWREF,
        CONST_PSBSMP.V_COUNTER_SYNC_PULSE,
        # CONST_PSBSMP.V_SIGGEN_ENABLE,
        # CONST_PSBSMP.V_SIGGEN_TYPE,
        # CONST_PSBSMP.V_SIGGEN_NUM_CYCLES,
        # CONST_PSBSMP.V_SIGGEN_N,
        # CONST_PSBSMP.V_SIGGEN_FREQ,
        # CONST_PSBSMP.V_SIGGEN_AMPLITUDE,
        # CONST_PSBSMP.V_SIGGEN_OFFSET,
        # CONST_PSBSMP.V_SIGGEN_AUX_PARAM,
        # CONST_PSBSMP.V_WFMREF_SELECTED,
        # CONST_PSBSMP.V_WFMREF_SYNC_MODE,
        # CONST_PSBSMP.V_WFMREF_FREQUENCY,
        # CONST_PSBSMP.V_WFMREF_GAIN,
        # CONST_PSBSMP.V_WFMREF_OFFSET,
        # CONST_PSBSMP.V_WFMREF0_START,
        # CONST_PSBSMP.V_WFMREF0_END,
        # CONST_PSBSMP.V_WFMREF0_IDX,
        # CONST_PSBSMP.V_WFMREF1_START,
        # CONST_PSBSMP.V_WFMREF1_END,
        # CONST_PSBSMP.V_WFMREF1_IDX,
        # CONST_PSBSMP.V_SCOPE_FREQUENCY,
        # CONST_PSBSMP.V_SCOPE_DURATION,
        # CONST_PSBSMP.V_SCOPE_SRC_DATA,
        # --- FBP_DCLink variables ---
        CONST_PSBSMP.V_PS_SOFT_INTERLOCKS,
        CONST_PSBSMP.V_PS_HARD_INTERLOCKS,
        CONST_PSBSMP.V_MODULES_STATUS,
        CONST_PSBSMP.V_V_OUT,
        CONST_PSBSMP.V_V_OUT_1,
        CONST_PSBSMP.V_V_OUT_2,
        CONST_PSBSMP.V_V_OUT_3,
        CONST_PSBSMP.V_DIG_POT_TAP,)
    groups[_const.G_SCAN] = (
        # --- common variables
        CONST_PSBSMP.V_PS_STATUS,
        CONST_PSBSMP.V_PS_SETPOINT,
        CONST_PSBSMP.V_PS_REFERENCE,
        CONST_PSBSMP.V_FIRMWARE_VERSION,
        CONST_PSBSMP.V_COUNTER_SET_SLOWREF,
        CONST_PSBSMP.V_COUNTER_SYNC_PULSE,
        # CONST_PSBSMP.V_SIGGEN_ENABLE,
        # CONST_PSBSMP.V_SIGGEN_TYPE,
        # CONST_PSBSMP.V_SIGGEN_NUM_CYCLES,
        # CONST_PSBSMP.V_SIGGEN_N,
        # CONST_PSBSMP.V_SIGGEN_FREQ,
        # CONST_PSBSMP.V_SIGGEN_AMPLITUDE,
        # CONST_PSBSMP.V_SIGGEN_OFFSET,
        # CONST_PSBSMP.V_SIGGEN_AUX_PARAM,
        # CONST_PSBSMP.V_WFMREF_SELECTED,
        # CONST_PSBSMP.V_WFMREF_SYNC_MODE,
        # CONST_PSBSMP.V_WFMREF_FREQUENCY,
        # CONST_PSBSMP.V_WFMREF_GAIN,
        # CONST_PSBSMP.V_WFMREF_OFFSET,
        # CONST_PSBSMP.V_WFMREF0_START,
        # CONST_PSBSMP.V_WFMREF0_END,
        # CONST_PSBSMP.V_WFMREF0_IDX,
        # CONST_PSBSMP.V_WFMREF1_START,
        # CONST_PSBSMP.V_WFMREF1_END,
        # CONST_PSBSMP.V_WFMREF1_IDX,
        # CONST_PSBSMP.V_SCOPE_FREQUENCY,
        # CONST_PSBSMP.V_SCOPE_DURATION,
        # CONST_PSBSMP.V_SCOPE_SRC_DATA,
        # --- FBP_DCLink variables ---
        CONST_PSBSMP.V_PS_SOFT_INTERLOCKS,
        CONST_PSBSMP.V_PS_HARD_INTERLOCKS,
        CONST_PSBSMP.V_MODULES_STATUS,
        CONST_PSBSMP.V_V_OUT,
        CONST_PSBSMP.V_V_OUT_1,
        CONST_PSBSMP.V_V_OUT_2,
        CONST_PSBSMP.V_V_OUT_3,
        CONST_PSBSMP.V_DIG_POT_TAP,)


class PRUCParmsFAC_2S_DCDC:
    """FAC_2S specific PRUC parameters.

    Represent FAC_2S_DCDC psmodels.
    """

    FREQ_SCAN = 5.0  # [Hz]

    # PS model parms
    model = _PSModelFactory.create('FAC_2S_DCDC')
    CONST_PSBSMP = model.bsmp_constants
    Entities = model.entities

    groups = dict()
    # reserved variable groups (not to be used)
    groups[_const.G_ALL] = tuple(sorted(Entities.list_variables(0)))
    groups[_const.G_READONLY] = tuple(sorted(Entities.list_variables(1)))
    groups[_const.G_WRITE] = tuple(sorted(Entities.list_variables(2)))
    # new variable groups useful for PRUController.
    groups[_const.G_ALLRELEVANT] = (
        # --- common variables
        CONST_PSBSMP.V_PS_STATUS,
        CONST_PSBSMP.V_PS_SETPOINT,
        CONST_PSBSMP.V_PS_REFERENCE,
        CONST_PSBSMP.V_FIRMWARE_VERSION,
        CONST_PSBSMP.V_COUNTER_SET_SLOWREF,
        CONST_PSBSMP.V_COUNTER_SYNC_PULSE,
        CONST_PSBSMP.V_SIGGEN_ENABLE,
        CONST_PSBSMP.V_SIGGEN_TYPE,
        CONST_PSBSMP.V_SIGGEN_NUM_CYCLES,
        CONST_PSBSMP.V_SIGGEN_N,
        CONST_PSBSMP.V_SIGGEN_FREQ,
        CONST_PSBSMP.V_SIGGEN_AMPLITUDE,
        CONST_PSBSMP.V_SIGGEN_OFFSET,
        CONST_PSBSMP.V_SIGGEN_AUX_PARAM,
        CONST_PSBSMP.V_WFMREF_SELECTED,
        CONST_PSBSMP.V_WFMREF_SYNC_MODE,
        CONST_PSBSMP.V_WFMREF_FREQUENCY,
        CONST_PSBSMP.V_WFMREF_GAIN,
        CONST_PSBSMP.V_WFMREF_OFFSET,
        CONST_PSBSMP.V_WFMREF0_START,
        CONST_PSBSMP.V_WFMREF0_END,
        CONST_PSBSMP.V_WFMREF0_IDX,
        CONST_PSBSMP.V_WFMREF1_START,
        CONST_PSBSMP.V_WFMREF1_END,
        CONST_PSBSMP.V_WFMREF1_IDX,
        CONST_PSBSMP.V_SCOPE_FREQUENCY,
        CONST_PSBSMP.V_SCOPE_DURATION,
        CONST_PSBSMP.V_SCOPE_SRC_DATA,
        # --- FAC_2S_DCDC variables ---
        CONST_PSBSMP.V_PS_SOFT_INTERLOCKS,
        CONST_PSBSMP.V_PS_HARD_INTERLOCKS,
        CONST_PSBSMP.V_I_LOAD_MEAN,
        CONST_PSBSMP.V_I_LOAD1,
        CONST_PSBSMP.V_I_LOAD2,
        CONST_PSBSMP.V_V_CAPBANK_1,
        CONST_PSBSMP.V_V_CAPBANK_2,
        CONST_PSBSMP.V_DUTY_CYCLE_1,
        CONST_PSBSMP.V_DUTY_CYCLE_2,
        CONST_PSBSMP.V_V_INPUT_IIB_1,
        CONST_PSBSMP.V_I_INPUT_IIB_1,
        CONST_PSBSMP.V_I_OUTPUT_IIB_1,
        CONST_PSBSMP.V_TEMP_IGBTS_1_IIB_1,
        CONST_PSBSMP.V_TEMP_IGBTS_2_IIB_1,
        CONST_PSBSMP.V_TEMP_INDUCTOR_IIB_1,
        CONST_PSBSMP.V_TEMP_HEATSINK_IIB_1,
        CONST_PSBSMP.V_V_DRIVER_IIB_1,
        CONST_PSBSMP.V_I_DRIVER_1_IIB_1,
        CONST_PSBSMP.V_I_DRIVER_2_IIB_1,
        CONST_PSBSMP.V_TEMP_BOARD_IIB_1,
        CONST_PSBSMP.V_RH_IIB_1,
        CONST_PSBSMP.V_IIB_INTERLOCKS_1,
        CONST_PSBSMP.V_IIB_ALARMS_1,
        CONST_PSBSMP.V_V_INPUT_IIB_2,
        CONST_PSBSMP.V_I_INPUT_IIB_2,
        CONST_PSBSMP.V_I_OUTPUT_IIB_2,
        CONST_PSBSMP.V_TEMP_IGBTS_1_IIB_2,
        CONST_PSBSMP.V_TEMP_IGBTS_2_IIB_2,
        CONST_PSBSMP.V_TEMP_INDUCTOR_IIB_2,
        CONST_PSBSMP.V_TEMP_HEATSINK_IIB_2,
        CONST_PSBSMP.V_V_DRIVER_IIB_2,
        CONST_PSBSMP.V_I_DRIVER_1_IIB_2,
        CONST_PSBSMP.V_I_DRIVER_2_IIB_2,
        CONST_PSBSMP.V_TEMP_BOARD_IIB_2,
        CONST_PSBSMP.V_RH_IIB_2,
        CONST_PSBSMP.V_IIB_INTERLOCKS_2,
        CONST_PSBSMP.V_IIB_ALARMS_2)
    groups[_const.G_SCAN] = (
        # --- common variables
        CONST_PSBSMP.V_PS_STATUS,
        CONST_PSBSMP.V_PS_SETPOINT,
        CONST_PSBSMP.V_PS_REFERENCE,
        # CONST_PSBSMP.V_FIRMWARE_VERSION,
        CONST_PSBSMP.V_COUNTER_SET_SLOWREF,
        CONST_PSBSMP.V_COUNTER_SYNC_PULSE,
        CONST_PSBSMP.V_SIGGEN_ENABLE,
        CONST_PSBSMP.V_SIGGEN_TYPE,
        CONST_PSBSMP.V_SIGGEN_NUM_CYCLES,
        CONST_PSBSMP.V_SIGGEN_N,
        CONST_PSBSMP.V_SIGGEN_FREQ,
        CONST_PSBSMP.V_SIGGEN_AMPLITUDE,
        CONST_PSBSMP.V_SIGGEN_OFFSET,
        CONST_PSBSMP.V_SIGGEN_AUX_PARAM,
        CONST_PSBSMP.V_WFMREF_SELECTED,
        CONST_PSBSMP.V_WFMREF_SYNC_MODE,
        CONST_PSBSMP.V_WFMREF_FREQUENCY,
        CONST_PSBSMP.V_WFMREF_GAIN,
        CONST_PSBSMP.V_WFMREF_OFFSET,
        CONST_PSBSMP.V_WFMREF0_START,
        CONST_PSBSMP.V_WFMREF0_END,
        CONST_PSBSMP.V_WFMREF0_IDX,
        CONST_PSBSMP.V_WFMREF1_START,
        CONST_PSBSMP.V_WFMREF1_END,
        CONST_PSBSMP.V_WFMREF1_IDX,
        CONST_PSBSMP.V_SCOPE_FREQUENCY,
        CONST_PSBSMP.V_SCOPE_DURATION,
        CONST_PSBSMP.V_SCOPE_SRC_DATA,
        # --- FAC_2S_DCDC variables ---
        CONST_PSBSMP.V_PS_SOFT_INTERLOCKS,
        CONST_PSBSMP.V_PS_HARD_INTERLOCKS,
        CONST_PSBSMP.V_I_LOAD_MEAN,
        CONST_PSBSMP.V_I_LOAD1,
        CONST_PSBSMP.V_I_LOAD2,
        CONST_PSBSMP.V_V_CAPBANK_1,
        CONST_PSBSMP.V_V_CAPBANK_2,
        CONST_PSBSMP.V_DUTY_CYCLE_1,
        CONST_PSBSMP.V_DUTY_CYCLE_2,
        CONST_PSBSMP.V_V_INPUT_IIB_1,
        CONST_PSBSMP.V_I_INPUT_IIB_1,
        CONST_PSBSMP.V_I_OUTPUT_IIB_1,
        CONST_PSBSMP.V_TEMP_IGBTS_1_IIB_1,
        CONST_PSBSMP.V_TEMP_IGBTS_2_IIB_1,
        CONST_PSBSMP.V_TEMP_INDUCTOR_IIB_1,
        CONST_PSBSMP.V_TEMP_HEATSINK_IIB_1,
        CONST_PSBSMP.V_V_DRIVER_IIB_1,
        CONST_PSBSMP.V_I_DRIVER_1_IIB_1,
        CONST_PSBSMP.V_I_DRIVER_2_IIB_1,
        CONST_PSBSMP.V_TEMP_BOARD_IIB_1,
        CONST_PSBSMP.V_RH_IIB_1,
        CONST_PSBSMP.V_IIB_INTERLOCKS_1,
        CONST_PSBSMP.V_IIB_ALARMS_1,
        CONST_PSBSMP.V_V_INPUT_IIB_2,
        CONST_PSBSMP.V_I_INPUT_IIB_2,
        CONST_PSBSMP.V_I_OUTPUT_IIB_2,
        CONST_PSBSMP.V_TEMP_IGBTS_1_IIB_2,
        CONST_PSBSMP.V_TEMP_IGBTS_2_IIB_2,
        CONST_PSBSMP.V_TEMP_INDUCTOR_IIB_2,
        CONST_PSBSMP.V_TEMP_HEATSINK_IIB_2,
        CONST_PSBSMP.V_V_DRIVER_IIB_2,
        CONST_PSBSMP.V_I_DRIVER_1_IIB_2,
        CONST_PSBSMP.V_I_DRIVER_2_IIB_2,
        CONST_PSBSMP.V_TEMP_BOARD_IIB_2,
        CONST_PSBSMP.V_RH_IIB_2,
        CONST_PSBSMP.V_IIB_INTERLOCKS_2,
        CONST_PSBSMP.V_IIB_ALARMS_2)


class PRUCParmsFAC_2P4S_DCDC:
    """FAC-specific PRUC parameters.

    Represent FAC_2P4S psmodels.
    """

    FREQ_SCAN = 5.0  # [Hz]

    # PS model parms
    model = _PSModelFactory.create('FAC_2P4S_DCDC')
    CONST_PSBSMP = model.bsmp_constants
    Entities = model.entities

    groups = dict()
    # reserved variable groups (not to be used)
    groups[_const.G_ALL] = tuple(sorted(Entities.list_variables(0)))
    groups[_const.G_READONLY] = tuple(sorted(Entities.list_variables(1)))
    groups[_const.G_WRITE] = tuple(sorted(Entities.list_variables(2)))
    # new variable groups useful for PRUController.
    groups[_const.G_ALLRELEVANT] = (
        # --- common variables
        CONST_PSBSMP.V_PS_STATUS,
        CONST_PSBSMP.V_PS_SETPOINT,
        CONST_PSBSMP.V_PS_REFERENCE,
        CONST_PSBSMP.V_FIRMWARE_VERSION,
        CONST_PSBSMP.V_COUNTER_SET_SLOWREF,
        CONST_PSBSMP.V_COUNTER_SYNC_PULSE,
        CONST_PSBSMP.V_SIGGEN_ENABLE,
        CONST_PSBSMP.V_SIGGEN_TYPE,
        CONST_PSBSMP.V_SIGGEN_NUM_CYCLES,
        CONST_PSBSMP.V_SIGGEN_N,
        CONST_PSBSMP.V_SIGGEN_FREQ,
        CONST_PSBSMP.V_SIGGEN_AMPLITUDE,
        CONST_PSBSMP.V_SIGGEN_OFFSET,
        CONST_PSBSMP.V_SIGGEN_AUX_PARAM,
        CONST_PSBSMP.V_WFMREF_SELECTED,
        CONST_PSBSMP.V_WFMREF_SYNC_MODE,
        CONST_PSBSMP.V_WFMREF_FREQUENCY,
        CONST_PSBSMP.V_WFMREF_GAIN,
        CONST_PSBSMP.V_WFMREF_OFFSET,
        CONST_PSBSMP.V_WFMREF0_START,
        CONST_PSBSMP.V_WFMREF0_END,
        CONST_PSBSMP.V_WFMREF0_IDX,
        CONST_PSBSMP.V_WFMREF1_START,
        CONST_PSBSMP.V_WFMREF1_END,
        CONST_PSBSMP.V_WFMREF1_IDX,
        CONST_PSBSMP.V_SCOPE_FREQUENCY,
        CONST_PSBSMP.V_SCOPE_DURATION,
        CONST_PSBSMP.V_SCOPE_SRC_DATA,
        # --- FAC_2P4S_DCDC variables ---
        CONST_PSBSMP.V_PS_SOFT_INTERLOCKS,
        CONST_PSBSMP.V_PS_HARD_INTERLOCKS,
        CONST_PSBSMP.V_I_LOAD_MEAN,
        CONST_PSBSMP.V_I_LOAD1,
        CONST_PSBSMP.V_I_LOAD2,
        CONST_PSBSMP.V_I_ARM_1,
        CONST_PSBSMP.V_I_ARM_2,
        CONST_PSBSMP.V_V_CAPBANK_1,
        CONST_PSBSMP.V_V_CAPBANK_2,
        CONST_PSBSMP.V_V_CAPBANK_3,
        CONST_PSBSMP.V_V_CAPBANK_4,
        CONST_PSBSMP.V_V_CAPBANK_5,
        CONST_PSBSMP.V_V_CAPBANK_6,
        CONST_PSBSMP.V_V_CAPBANK_7,
        CONST_PSBSMP.V_V_CAPBANK_8,
        CONST_PSBSMP.V_DUTY_CYCLE_1,
        CONST_PSBSMP.V_DUTY_CYCLE_2,
        CONST_PSBSMP.V_DUTY_CYCLE_3,
        CONST_PSBSMP.V_DUTY_CYCLE_4,
        CONST_PSBSMP.V_DUTY_CYCLE_5,
        CONST_PSBSMP.V_DUTY_CYCLE_6,
        CONST_PSBSMP.V_DUTY_CYCLE_7,
        CONST_PSBSMP.V_DUTY_CYCLE_8,
        CONST_PSBSMP.V_V_INPUT_IIB_A,
        CONST_PSBSMP.V_I_INPUT_IIB_A,
        CONST_PSBSMP.V_I_OUTPUT_IIB_A,
        CONST_PSBSMP.V_TEMP_IGBTS_1_IIB_A,
        CONST_PSBSMP.V_TEMP_IGBTS_2_IIB_A,
        CONST_PSBSMP.V_TEMP_INDUCTOR_IIB_A,
        CONST_PSBSMP.V_TEMP_HEATSINK_IIB_A,
        CONST_PSBSMP.V_V_DRIVER_IIB_A,
        CONST_PSBSMP.V_I_DRIVER_1_IIB_A,
        CONST_PSBSMP.V_I_DRIVER_2_IIB_A,
        CONST_PSBSMP.V_TEMP_BOARD_IIB_A,
        CONST_PSBSMP.V_RH_IIB_A,
        CONST_PSBSMP.V_IIB_INTERLOCKS_A,
        CONST_PSBSMP.V_IIB_ALARMS_A,
        CONST_PSBSMP.V_V_INPUT_IIB_B,
        CONST_PSBSMP.V_I_INPUT_IIB_B,
        CONST_PSBSMP.V_I_OUTPUT_IIB_B,
        CONST_PSBSMP.V_TEMP_IGBTS_1_IIB_B,
        CONST_PSBSMP.V_TEMP_IGBTS_2_IIB_B,
        CONST_PSBSMP.V_TEMP_INDUCTOR_IIB_B,
        CONST_PSBSMP.V_TEMP_HEATSINK_IIB_B,
        CONST_PSBSMP.V_V_DRIVER_IIB_B,
        CONST_PSBSMP.V_I_DRIVER_1_IIB_B,
        CONST_PSBSMP.V_I_DRIVER_2_IIB_B,
        CONST_PSBSMP.V_TEMP_BOARD_IIB_B,
        CONST_PSBSMP.V_RH_IIB_B,
        CONST_PSBSMP.V_IIB_INTERLOCKS_B,
        CONST_PSBSMP.V_IIB_ALARMS_B,)
    groups[_const.G_SCAN] = (
        # --- common variables
        CONST_PSBSMP.V_PS_STATUS,
        CONST_PSBSMP.V_PS_SETPOINT,
        CONST_PSBSMP.V_PS_REFERENCE,
        # CONST_PSBSMP.V_FIRMWARE_VERSION,
        CONST_PSBSMP.V_COUNTER_SET_SLOWREF,
        CONST_PSBSMP.V_COUNTER_SYNC_PULSE,
        CONST_PSBSMP.V_SIGGEN_ENABLE,
        CONST_PSBSMP.V_SIGGEN_TYPE,
        CONST_PSBSMP.V_SIGGEN_NUM_CYCLES,
        CONST_PSBSMP.V_SIGGEN_N,
        CONST_PSBSMP.V_SIGGEN_FREQ,
        CONST_PSBSMP.V_SIGGEN_AMPLITUDE,
        CONST_PSBSMP.V_SIGGEN_OFFSET,
        CONST_PSBSMP.V_SIGGEN_AUX_PARAM,
        CONST_PSBSMP.V_WFMREF_SELECTED,
        CONST_PSBSMP.V_WFMREF_SYNC_MODE,
        CONST_PSBSMP.V_WFMREF_FREQUENCY,
        CONST_PSBSMP.V_WFMREF_GAIN,
        CONST_PSBSMP.V_WFMREF_OFFSET,
        CONST_PSBSMP.V_WFMREF0_START,
        CONST_PSBSMP.V_WFMREF0_END,
        CONST_PSBSMP.V_WFMREF0_IDX,
        CONST_PSBSMP.V_WFMREF1_START,
        CONST_PSBSMP.V_WFMREF1_END,
        CONST_PSBSMP.V_WFMREF1_IDX,
        CONST_PSBSMP.V_SCOPE_FREQUENCY,
        CONST_PSBSMP.V_SCOPE_DURATION,
        CONST_PSBSMP.V_SCOPE_SRC_DATA,
        # --- FAC_2P4S_DCDC variables ---
        CONST_PSBSMP.V_PS_SOFT_INTERLOCKS,
        CONST_PSBSMP.V_PS_HARD_INTERLOCKS,
        CONST_PSBSMP.V_I_LOAD_MEAN,
        CONST_PSBSMP.V_I_LOAD1,
        CONST_PSBSMP.V_I_LOAD2,
        CONST_PSBSMP.V_I_ARM_1,
        CONST_PSBSMP.V_I_ARM_2,
        CONST_PSBSMP.V_V_CAPBANK_1,
        CONST_PSBSMP.V_V_CAPBANK_2,
        CONST_PSBSMP.V_V_CAPBANK_3,
        CONST_PSBSMP.V_V_CAPBANK_4,
        CONST_PSBSMP.V_V_CAPBANK_5,
        CONST_PSBSMP.V_V_CAPBANK_6,
        CONST_PSBSMP.V_V_CAPBANK_7,
        CONST_PSBSMP.V_V_CAPBANK_8,
        CONST_PSBSMP.V_DUTY_CYCLE_1,
        CONST_PSBSMP.V_DUTY_CYCLE_2,
        CONST_PSBSMP.V_DUTY_CYCLE_3,
        CONST_PSBSMP.V_DUTY_CYCLE_4,
        CONST_PSBSMP.V_DUTY_CYCLE_5,
        CONST_PSBSMP.V_DUTY_CYCLE_6,
        CONST_PSBSMP.V_DUTY_CYCLE_7,
        CONST_PSBSMP.V_DUTY_CYCLE_8,
        CONST_PSBSMP.V_V_INPUT_IIB_A,
        CONST_PSBSMP.V_I_INPUT_IIB_A,
        CONST_PSBSMP.V_I_OUTPUT_IIB_A,
        CONST_PSBSMP.V_TEMP_IGBTS_1_IIB_A,
        CONST_PSBSMP.V_TEMP_IGBTS_2_IIB_A,
        CONST_PSBSMP.V_TEMP_INDUCTOR_IIB_A,
        CONST_PSBSMP.V_TEMP_HEATSINK_IIB_A,
        CONST_PSBSMP.V_V_DRIVER_IIB_A,
        CONST_PSBSMP.V_I_DRIVER_1_IIB_A,
        CONST_PSBSMP.V_I_DRIVER_2_IIB_A,
        CONST_PSBSMP.V_TEMP_BOARD_IIB_A,
        CONST_PSBSMP.V_RH_IIB_A,
        CONST_PSBSMP.V_IIB_INTERLOCKS_A,
        CONST_PSBSMP.V_IIB_ALARMS_A,
        CONST_PSBSMP.V_V_INPUT_IIB_B,
        CONST_PSBSMP.V_I_INPUT_IIB_B,
        CONST_PSBSMP.V_I_OUTPUT_IIB_B,
        CONST_PSBSMP.V_TEMP_IGBTS_1_IIB_B,
        CONST_PSBSMP.V_TEMP_IGBTS_2_IIB_B,
        CONST_PSBSMP.V_TEMP_INDUCTOR_IIB_B,
        CONST_PSBSMP.V_TEMP_HEATSINK_IIB_B,
        CONST_PSBSMP.V_V_DRIVER_IIB_B,
        CONST_PSBSMP.V_I_DRIVER_1_IIB_B,
        CONST_PSBSMP.V_I_DRIVER_2_IIB_B,
        CONST_PSBSMP.V_TEMP_BOARD_IIB_B,
        CONST_PSBSMP.V_RH_IIB_B,
        CONST_PSBSMP.V_IIB_INTERLOCKS_B,
        CONST_PSBSMP.V_IIB_ALARMS_B,)


class PRUCParmsFAC_DCDC:
    """FAC-specific PRUC parameters.

    Represent FAC, FAC_2S, FAC_2P4S psmodels.
    """

    FREQ_SCAN = 5.0  # [Hz]

    # PS model parms
    model = _PSModelFactory.create('FAC_DCDC')
    CONST_PSBSMP = model.bsmp_constants
    Entities = model.entities

    groups = dict()
    # reserved variable groups (not to be used)
    groups[_const.G_ALL] = tuple(sorted(Entities.list_variables(0)))
    groups[_const.G_READONLY] = tuple(sorted(Entities.list_variables(1)))
    groups[_const.G_WRITE] = tuple(sorted(Entities.list_variables(2)))
    # new variable groups useful for PRUController.
    groups[_const.G_ALLRELEVANT] = (
        # --- common variables
        CONST_PSBSMP.V_PS_STATUS,
        CONST_PSBSMP.V_PS_SETPOINT,
        CONST_PSBSMP.V_PS_REFERENCE,
        CONST_PSBSMP.V_FIRMWARE_VERSION,
        CONST_PSBSMP.V_COUNTER_SET_SLOWREF,
        CONST_PSBSMP.V_COUNTER_SYNC_PULSE,
        CONST_PSBSMP.V_SIGGEN_ENABLE,
        CONST_PSBSMP.V_SIGGEN_TYPE,
        CONST_PSBSMP.V_SIGGEN_NUM_CYCLES,
        CONST_PSBSMP.V_SIGGEN_N,
        CONST_PSBSMP.V_SIGGEN_FREQ,
        CONST_PSBSMP.V_SIGGEN_AMPLITUDE,
        CONST_PSBSMP.V_SIGGEN_OFFSET,
        CONST_PSBSMP.V_SIGGEN_AUX_PARAM,
        CONST_PSBSMP.V_WFMREF_SELECTED,
        CONST_PSBSMP.V_WFMREF_SYNC_MODE,
        CONST_PSBSMP.V_WFMREF_FREQUENCY,
        CONST_PSBSMP.V_WFMREF_GAIN,
        CONST_PSBSMP.V_WFMREF_OFFSET,
        CONST_PSBSMP.V_WFMREF0_START,
        CONST_PSBSMP.V_WFMREF0_END,
        CONST_PSBSMP.V_WFMREF0_IDX,
        CONST_PSBSMP.V_WFMREF1_START,
        CONST_PSBSMP.V_WFMREF1_END,
        CONST_PSBSMP.V_WFMREF1_IDX,
        CONST_PSBSMP.V_SCOPE_FREQUENCY,
        CONST_PSBSMP.V_SCOPE_DURATION,
        CONST_PSBSMP.V_SCOPE_SRC_DATA,
        # --- FAC variables ---
        CONST_PSBSMP.V_PS_SOFT_INTERLOCKS,
        CONST_PSBSMP.V_PS_HARD_INTERLOCKS,
        CONST_PSBSMP.V_I_LOAD_MEAN,
        CONST_PSBSMP.V_I_LOAD1,
        CONST_PSBSMP.V_I_LOAD2,
        CONST_PSBSMP.V_V_CAPBANK,
        CONST_PSBSMP.V_DUTY_CYCLE,
        CONST_PSBSMP.V_V_INPUT_IIB,
        CONST_PSBSMP.V_I_INPUT_IIB,
        CONST_PSBSMP.V_I_OUTPUT_IIB,
        CONST_PSBSMP.V_TEMP_IGBT_1_IIB,
        CONST_PSBSMP.V_TEMP_IGBT_2_IIB,
        CONST_PSBSMP.V_TEMP_INDUCTOR_IIB,
        CONST_PSBSMP.V_TEMP_HEATSINK_IIB,
        CONST_PSBSMP.V_V_DRIVER_IIB,
        CONST_PSBSMP.V_I_DRIVER_1_IIB,
        CONST_PSBSMP.V_I_DRIVER_2_IIB,
        CONST_PSBSMP.V_I_LEAKAGE_IIB,
        CONST_PSBSMP.V_TEMP_BOARD_IIB,
        CONST_PSBSMP.V_RH_IIB,
        CONST_PSBSMP.V_IIB_INTERLOCKS,
        CONST_PSBSMP.V_IIB_ALARMS,)
    groups[_const.G_SCAN] = (
        # --- common variables
        CONST_PSBSMP.V_PS_STATUS,
        CONST_PSBSMP.V_PS_SETPOINT,
        CONST_PSBSMP.V_PS_REFERENCE,
        # CONST_PSBSMP.V_FIRMWARE_VERSION,
        CONST_PSBSMP.V_COUNTER_SET_SLOWREF,
        CONST_PSBSMP.V_COUNTER_SYNC_PULSE,
        CONST_PSBSMP.V_SIGGEN_ENABLE,
        CONST_PSBSMP.V_SIGGEN_TYPE,
        CONST_PSBSMP.V_SIGGEN_NUM_CYCLES,
        CONST_PSBSMP.V_SIGGEN_N,
        CONST_PSBSMP.V_SIGGEN_FREQ,
        CONST_PSBSMP.V_SIGGEN_AMPLITUDE,
        CONST_PSBSMP.V_SIGGEN_OFFSET,
        CONST_PSBSMP.V_SIGGEN_AUX_PARAM,
        CONST_PSBSMP.V_WFMREF_SELECTED,
        CONST_PSBSMP.V_WFMREF_SYNC_MODE,
        CONST_PSBSMP.V_WFMREF_FREQUENCY,
        CONST_PSBSMP.V_WFMREF_GAIN,
        CONST_PSBSMP.V_WFMREF_OFFSET,
        CONST_PSBSMP.V_WFMREF0_START,
        CONST_PSBSMP.V_WFMREF0_END,
        CONST_PSBSMP.V_WFMREF0_IDX,
        CONST_PSBSMP.V_WFMREF1_START,
        CONST_PSBSMP.V_WFMREF1_END,
        CONST_PSBSMP.V_WFMREF1_IDX,
        CONST_PSBSMP.V_SCOPE_FREQUENCY,
        CONST_PSBSMP.V_SCOPE_DURATION,
        CONST_PSBSMP.V_SCOPE_SRC_DATA,
        # --- FAC variables ---
        CONST_PSBSMP.V_PS_SOFT_INTERLOCKS,
        CONST_PSBSMP.V_PS_HARD_INTERLOCKS,
        CONST_PSBSMP.V_I_LOAD_MEAN,
        CONST_PSBSMP.V_I_LOAD1,
        CONST_PSBSMP.V_I_LOAD2,
        CONST_PSBSMP.V_V_CAPBANK,
        CONST_PSBSMP.V_DUTY_CYCLE,
        CONST_PSBSMP.V_V_INPUT_IIB,
        CONST_PSBSMP.V_I_INPUT_IIB,
        CONST_PSBSMP.V_I_OUTPUT_IIB,
        CONST_PSBSMP.V_TEMP_IGBT_1_IIB,
        CONST_PSBSMP.V_TEMP_IGBT_2_IIB,
        CONST_PSBSMP.V_TEMP_INDUCTOR_IIB,
        CONST_PSBSMP.V_TEMP_HEATSINK_IIB,
        CONST_PSBSMP.V_V_DRIVER_IIB,
        CONST_PSBSMP.V_I_DRIVER_1_IIB,
        CONST_PSBSMP.V_I_DRIVER_2_IIB,
        CONST_PSBSMP.V_I_LEAKAGE_IIB,
        CONST_PSBSMP.V_TEMP_BOARD_IIB,
        CONST_PSBSMP.V_RH_IIB,
        CONST_PSBSMP.V_IIB_INTERLOCKS,
        CONST_PSBSMP.V_IIB_ALARMS,)


class PRUCParmsFAC_2S_ACDC:
    """FAC_2S_ACDC-specific PRUC parameters."""

    FREQ_SCAN = 2.0  # [Hz]

    # PS model parms
    model = _PSModelFactory.create('FAC_2S_ACDC')
    CONST_PSBSMP = model.bsmp_constants
    Entities = model.entities

    groups = dict()
    # reserved variable groups (not to be used)
    groups[_const.G_ALL] = tuple(sorted(Entities.list_variables(0)))
    groups[_const.G_READONLY] = tuple(sorted(Entities.list_variables(1)))
    groups[_const.G_WRITE] = tuple(sorted(Entities.list_variables(2)))
    # new variable groups useful for PRUController.
    groups[_const.G_ALLRELEVANT] = (
        # --- common variables
        CONST_PSBSMP.V_PS_STATUS,
        CONST_PSBSMP.V_PS_SETPOINT,
        CONST_PSBSMP.V_PS_REFERENCE,
        CONST_PSBSMP.V_FIRMWARE_VERSION,
        CONST_PSBSMP.V_COUNTER_SET_SLOWREF,
        CONST_PSBSMP.V_COUNTER_SYNC_PULSE,
        # CONST_PSBSMP.V_SIGGEN_ENABLE,
        # CONST_PSBSMP.V_SIGGEN_TYPE,
        # CONST_PSBSMP.V_SIGGEN_NUM_CYCLES,
        # CONST_PSBSMP.V_SIGGEN_N,
        # CONST_PSBSMP.V_SIGGEN_FREQ,
        # CONST_PSBSMP.V_SIGGEN_AMPLITUDE,
        # CONST_PSBSMP.V_SIGGEN_OFFSET,
        # CONST_PSBSMP.V_SIGGEN_AUX_PARAM,
        # CONST_PSBSMP.V_WFMREF_SELECTED,
        # CONST_PSBSMP.V_WFMREF_SYNC_MODE,
        # CONST_PSBSMP.V_WFMREF_FREQUENCY,
        # CONST_PSBSMP.V_WFMREF_GAIN,
        # CONST_PSBSMP.V_WFMREF_OFFSET,
        # CONST_PSBSMP.V_WFMREF0_START,
        # CONST_PSBSMP.V_WFMREF0_END,
        # CONST_PSBSMP.V_WFMREF0_IDX,
        # CONST_PSBSMP.V_WFMREF1_START,
        # CONST_PSBSMP.V_WFMREF1_END,
        # CONST_PSBSMP.V_WFMREF1_IDX,
        # CONST_PSBSMP.V_SCOPE_FREQUENCY,
        # CONST_PSBSMP.V_SCOPE_DURATION,
        # CONST_PSBSMP.V_SCOPE_SRC_DATA,
        # --- FAC_2S_ACDC variables ---
        CONST_PSBSMP.V_PS_SOFT_INTERLOCKS,
        CONST_PSBSMP.V_PS_HARD_INTERLOCKS,
        CONST_PSBSMP.V_V_CAPBANK,
        CONST_PSBSMP.V_I_OUT_RECTIFIER,
        CONST_PSBSMP.V_DUTY_CYCLE,
        CONST_PSBSMP.V_I_INPUT_IS_IIB,
        CONST_PSBSMP.V_V_INPUT_IS_IIB,
        CONST_PSBSMP.V_TEMP_IGBT_IS_IIB,
        CONST_PSBSMP.V_V_DRIVER_IS_IIB,
        CONST_PSBSMP.V_I_DRIVER_IS_IIB,
        CONST_PSBSMP.V_TEMP_INDUCTOR_IS_IIB,
        CONST_PSBSMP.V_TEMP_HEATSINK_IS_IIB,
        CONST_PSBSMP.V_TEMP_BOARD_IS_IIB,
        CONST_PSBSMP.V_RH_IS_IIB,
        CONST_PSBSMP.V_IIB_INTERLOCKS_IS,
        CONST_PSBSMP.V_IIB_ALARMS_IS,
        CONST_PSBSMP.V_V_OUTPUT_CMD_IIB,
        CONST_PSBSMP.V_V_CAPBANK_CMD_IIB,
        CONST_PSBSMP.V_TEMP_RECT_INDUCTOR_CMD_IIB,
        CONST_PSBSMP.V_TEMP_RECT_HEATSINK_CMD_IIB,
        CONST_PSBSMP.V_V_EXT_BOARDS_CMD_IIB,
        CONST_PSBSMP.V_I_AUX_BOARD_CMD_IIB,
        CONST_PSBSMP.V_I_IDB_BOARD_CMD_IIB,
        CONST_PSBSMP.V_I_LEAKAGE_CMD_IIB,
        CONST_PSBSMP.V_TEMP_BOARD_CMD_IIB,
        CONST_PSBSMP.V_RH_CMD_IIB,
        CONST_PSBSMP.V_IIB_INTERLOCKS_CMD,
        CONST_PSBSMP.V_IIB_ALARMS_CMD,)
    groups[_const.G_SCAN] = (
        # --- common variables
        CONST_PSBSMP.V_PS_STATUS,
        CONST_PSBSMP.V_PS_SETPOINT,
        CONST_PSBSMP.V_PS_REFERENCE,
        # CONST_PSBSMP.V_FIRMWARE_VERSION,
        CONST_PSBSMP.V_COUNTER_SET_SLOWREF,
        CONST_PSBSMP.V_COUNTER_SYNC_PULSE,
        # CONST_PSBSMP.V_SIGGEN_ENABLE,
        # CONST_PSBSMP.V_SIGGEN_TYPE,
        # CONST_PSBSMP.V_SIGGEN_NUM_CYCLES,
        # CONST_PSBSMP.V_SIGGEN_N,
        # CONST_PSBSMP.V_SIGGEN_FREQ,
        # CONST_PSBSMP.V_SIGGEN_AMPLITUDE,
        # CONST_PSBSMP.V_SIGGEN_OFFSET,
        # CONST_PSBSMP.V_SIGGEN_AUX_PARAM,
        # CONST_PSBSMP.V_WFMREF_SELECTED,
        # CONST_PSBSMP.V_WFMREF_SYNC_MODE,
        # CONST_PSBSMP.V_WFMREF_FREQUENCY,
        # CONST_PSBSMP.V_WFMREF_GAIN,
        # CONST_PSBSMP.V_WFMREF_OFFSET,
        # CONST_PSBSMP.V_WFMREF0_START,
        # CONST_PSBSMP.V_WFMREF0_END,
        # CONST_PSBSMP.V_WFMREF0_IDX,
        # CONST_PSBSMP.V_WFMREF1_START,
        # CONST_PSBSMP.V_WFMREF1_END,
        # CONST_PSBSMP.V_WFMREF1_IDX,
        # CONST_PSBSMP.V_SCOPE_FREQUENCY,
        # CONST_PSBSMP.V_SCOPE_DURATION,
        # CONST_PSBSMP.V_SCOPE_SRC_DATA,
        # --- FAC_2S_ACDC variables ---
        CONST_PSBSMP.V_PS_SOFT_INTERLOCKS,
        CONST_PSBSMP.V_PS_HARD_INTERLOCKS,
        CONST_PSBSMP.V_V_CAPBANK,
        CONST_PSBSMP.V_I_OUT_RECTIFIER,
        CONST_PSBSMP.V_DUTY_CYCLE,
        CONST_PSBSMP.V_I_INPUT_IS_IIB,
        CONST_PSBSMP.V_V_INPUT_IS_IIB,
        CONST_PSBSMP.V_TEMP_IGBT_IS_IIB,
        CONST_PSBSMP.V_V_DRIVER_IS_IIB,
        CONST_PSBSMP.V_I_DRIVER_IS_IIB,
        CONST_PSBSMP.V_TEMP_INDUCTOR_IS_IIB,
        CONST_PSBSMP.V_TEMP_HEATSINK_IS_IIB,
        CONST_PSBSMP.V_TEMP_BOARD_IS_IIB,
        CONST_PSBSMP.V_RH_IS_IIB,
        CONST_PSBSMP.V_IIB_INTERLOCKS_IS,
        CONST_PSBSMP.V_IIB_ALARMS_IS,
        CONST_PSBSMP.V_V_OUTPUT_CMD_IIB,
        CONST_PSBSMP.V_V_CAPBANK_CMD_IIB,
        CONST_PSBSMP.V_TEMP_RECT_INDUCTOR_CMD_IIB,
        CONST_PSBSMP.V_TEMP_RECT_HEATSINK_CMD_IIB,
        CONST_PSBSMP.V_V_EXT_BOARDS_CMD_IIB,
        CONST_PSBSMP.V_I_AUX_BOARD_CMD_IIB,
        CONST_PSBSMP.V_I_IDB_BOARD_CMD_IIB,
        CONST_PSBSMP.V_I_LEAKAGE_CMD_IIB,
        CONST_PSBSMP.V_TEMP_BOARD_CMD_IIB,
        CONST_PSBSMP.V_RH_CMD_IIB,
        CONST_PSBSMP.V_IIB_INTERLOCKS_CMD,
        CONST_PSBSMP.V_IIB_ALARMS_CMD,)


class PRUCParmsFAC_2P4S_ACDC:
    """FAC_2P4S_ACDC-specific PRUC parameters."""

    FREQ_SCAN = 2.0  # [Hz]

    # PS model parms
    model = _PSModelFactory.create('FAC_2P4S_ACDC')
    CONST_PSBSMP = model.bsmp_constants
    Entities = model.entities

    groups = dict()
    # reserved variable groups (not to be used)
    groups[_const.G_ALL] = tuple(sorted(Entities.list_variables(0)))
    groups[_const.G_READONLY] = tuple(sorted(Entities.list_variables(1)))
    groups[_const.G_WRITE] = tuple(sorted(Entities.list_variables(2)))
    # new variable groups usefull for PRUController.
    groups[_const.G_ALLRELEVANT] = (
        # --- common variables
        CONST_PSBSMP.V_PS_STATUS,
        CONST_PSBSMP.V_PS_SETPOINT,
        CONST_PSBSMP.V_PS_REFERENCE,
        CONST_PSBSMP.V_FIRMWARE_VERSION,
        CONST_PSBSMP.V_COUNTER_SET_SLOWREF,
        CONST_PSBSMP.V_COUNTER_SYNC_PULSE,
        # CONST_PSBSMP.V_SIGGEN_ENABLE,
        # CONST_PSBSMP.V_SIGGEN_TYPE,
        # CONST_PSBSMP.V_SIGGEN_NUM_CYCLES,
        # CONST_PSBSMP.V_SIGGEN_N,
        # CONST_PSBSMP.V_SIGGEN_FREQ,
        # CONST_PSBSMP.V_SIGGEN_AMPLITUDE,
        # CONST_PSBSMP.V_SIGGEN_OFFSET,
        # CONST_PSBSMP.V_SIGGEN_AUX_PARAM,
        # CONST_PSBSMP.V_WFMREF_SELECTED,
        # CONST_PSBSMP.V_WFMREF_SYNC_MODE,
        # CONST_PSBSMP.V_WFMREF_FREQUENCY,
        # CONST_PSBSMP.V_WFMREF_GAIN,
        # CONST_PSBSMP.V_WFMREF_OFFSET,
        # CONST_PSBSMP.V_WFMREF0_START,
        # CONST_PSBSMP.V_WFMREF0_END,
        # CONST_PSBSMP.V_WFMREF0_IDX,
        # CONST_PSBSMP.V_WFMREF1_START,
        # CONST_PSBSMP.V_WFMREF1_END,
        # CONST_PSBSMP.V_WFMREF1_IDX,
        # CONST_PSBSMP.V_SCOPE_FREQUENCY,
        # CONST_PSBSMP.V_SCOPE_DURATION,
        # CONST_PSBSMP.V_SCOPE_SRC_DATA,
        # --- FAC_2P4S_ACDC variables ---
        CONST_PSBSMP.V_PS_SOFT_INTERLOCKS,
        CONST_PSBSMP.V_PS_HARD_INTERLOCKS,
        CONST_PSBSMP.V_V_CAPACITOR_BANK,
        CONST_PSBSMP.V_I_OUT_RECTIFIER,
        CONST_PSBSMP.V_DUTY_CYCLE,
        CONST_PSBSMP.V_I_INPUT_IS_IIB,
        CONST_PSBSMP.V_V_INPUT_IS_IIB,
        CONST_PSBSMP.V_TEMP_IGBT_IS_IIB,
        CONST_PSBSMP.V_V_DRIVER_IS_IIB,
        CONST_PSBSMP.V_I_DRIVER_IS_IIB,
        CONST_PSBSMP.V_TEMP_INDUCTOR_IS_IIB,
        CONST_PSBSMP.V_TEMP_HEATSINK_IS_IIB,
        CONST_PSBSMP.V_TEMP_BOARD_IS_IIB,
        CONST_PSBSMP.V_RH_IS_IIB,
        CONST_PSBSMP.V_IIB_INTERLOCKS_IS,
        CONST_PSBSMP.V_IIB_ALARMS_IS,
        CONST_PSBSMP.V_V_OUTPUT_CMD_IIB,
        CONST_PSBSMP.V_V_CAPBANK_CMD_IIB,
        CONST_PSBSMP.V_TEMP_RECT_INDUCTOR_CMD_IIB,
        CONST_PSBSMP.V_TEMP_RECT_HEATSINK_CMD_IIB,
        CONST_PSBSMP.V_V_EXT_BOARDS_CMD_IIB,
        CONST_PSBSMP.V_I_AUX_BOARD_CMD_IIB,
        CONST_PSBSMP.V_I_IDB_BOARD_CMD_IIB,
        CONST_PSBSMP.V_I_LEAKAGE_CMD_IIB,
        CONST_PSBSMP.V_TEMP_BOARD_CMD_IIB,
        CONST_PSBSMP.V_RH_CMD_IIB,
        CONST_PSBSMP.V_IIB_INTERLOCKS_CMD,
        CONST_PSBSMP.V_IIB_ALARMS_CMD,)
    groups[_const.G_SCAN] = (
        # --- common variables
        CONST_PSBSMP.V_PS_STATUS,
        CONST_PSBSMP.V_PS_SETPOINT,
        CONST_PSBSMP.V_PS_REFERENCE,
        # CONST_PSBSMP.V_FIRMWARE_VERSION,
        CONST_PSBSMP.V_COUNTER_SET_SLOWREF,
        CONST_PSBSMP.V_COUNTER_SYNC_PULSE,
        # CONST_PSBSMP.V_SIGGEN_ENABLE,
        # CONST_PSBSMP.V_SIGGEN_TYPE,
        # CONST_PSBSMP.V_SIGGEN_NUM_CYCLES,
        # CONST_PSBSMP.V_SIGGEN_N,
        # CONST_PSBSMP.V_SIGGEN_FREQ,
        # CONST_PSBSMP.V_SIGGEN_AMPLITUDE,
        # CONST_PSBSMP.V_SIGGEN_OFFSET,
        # CONST_PSBSMP.V_SIGGEN_AUX_PARAM,
        # CONST_PSBSMP.V_WFMREF_SELECTED,
        # CONST_PSBSMP.V_WFMREF_SYNC_MODE,
        # CONST_PSBSMP.V_WFMREF_FREQUENCY,
        # CONST_PSBSMP.V_WFMREF_GAIN,
        # CONST_PSBSMP.V_WFMREF_OFFSET,
        # CONST_PSBSMP.V_WFMREF0_START,
        # CONST_PSBSMP.V_WFMREF0_END,
        # CONST_PSBSMP.V_WFMREF0_IDX,
        # CONST_PSBSMP.V_WFMREF1_START,
        # CONST_PSBSMP.V_WFMREF1_END,
        # CONST_PSBSMP.V_WFMREF1_IDX,
        # CONST_PSBSMP.V_SCOPE_FREQUENCY,
        # CONST_PSBSMP.V_SCOPE_DURATION,
        # CONST_PSBSMP.V_SCOPE_SRC_DATA,
        # --- FAC_2P4S_ACDC variables ---
        CONST_PSBSMP.V_PS_SOFT_INTERLOCKS,
        CONST_PSBSMP.V_PS_HARD_INTERLOCKS,
        CONST_PSBSMP.V_V_CAPACITOR_BANK,
        CONST_PSBSMP.V_I_OUT_RECTIFIER,
        CONST_PSBSMP.V_DUTY_CYCLE,
        CONST_PSBSMP.V_I_INPUT_IS_IIB,
        CONST_PSBSMP.V_V_INPUT_IS_IIB,
        CONST_PSBSMP.V_TEMP_IGBT_IS_IIB,
        CONST_PSBSMP.V_V_DRIVER_IS_IIB,
        CONST_PSBSMP.V_I_DRIVER_IS_IIB,
        CONST_PSBSMP.V_TEMP_INDUCTOR_IS_IIB,
        CONST_PSBSMP.V_TEMP_HEATSINK_IS_IIB,
        CONST_PSBSMP.V_TEMP_BOARD_IS_IIB,
        CONST_PSBSMP.V_RH_IS_IIB,
        CONST_PSBSMP.V_IIB_INTERLOCKS_IS,
        CONST_PSBSMP.V_IIB_ALARMS_IS,
        CONST_PSBSMP.V_V_OUTPUT_CMD_IIB,
        CONST_PSBSMP.V_V_CAPBANK_CMD_IIB,
        CONST_PSBSMP.V_TEMP_RECT_INDUCTOR_CMD_IIB,
        CONST_PSBSMP.V_TEMP_RECT_HEATSINK_CMD_IIB,
        CONST_PSBSMP.V_V_EXT_BOARDS_CMD_IIB,
        CONST_PSBSMP.V_I_AUX_BOARD_CMD_IIB,
        CONST_PSBSMP.V_I_IDB_BOARD_CMD_IIB,
        CONST_PSBSMP.V_I_LEAKAGE_CMD_IIB,
        CONST_PSBSMP.V_TEMP_BOARD_CMD_IIB,
        CONST_PSBSMP.V_RH_CMD_IIB,
        CONST_PSBSMP.V_IIB_INTERLOCKS_CMD,
        CONST_PSBSMP.V_IIB_ALARMS_CMD,)


class PRUCParmsFAP:
    """FAC-specific PRUC parameters.

    Represent FAP
    """

    FREQ_SCAN = 5.0  # [Hz]

    # PS model parms
    model = _PSModelFactory.create('FAP')
    CONST_PSBSMP = model.bsmp_constants
    Entities = model.entities

    groups = dict()
    # reserved variable groups (not to be used)
    groups[_const.G_ALL] = tuple(sorted(Entities.list_variables(0)))
    groups[_const.G_READONLY] = tuple(sorted(Entities.list_variables(1)))
    groups[_const.G_WRITE] = tuple(sorted(Entities.list_variables(2)))
    # new variable groups useful for PRUController.
    groups[_const.G_ALLRELEVANT] = (
        # --- common variables
        CONST_PSBSMP.V_PS_STATUS,
        CONST_PSBSMP.V_PS_SETPOINT,
        CONST_PSBSMP.V_PS_REFERENCE,
        CONST_PSBSMP.V_FIRMWARE_VERSION,
        CONST_PSBSMP.V_COUNTER_SET_SLOWREF,
        CONST_PSBSMP.V_COUNTER_SYNC_PULSE,
        CONST_PSBSMP.V_SIGGEN_ENABLE,
        CONST_PSBSMP.V_SIGGEN_TYPE,
        CONST_PSBSMP.V_SIGGEN_NUM_CYCLES,
        CONST_PSBSMP.V_SIGGEN_N,
        CONST_PSBSMP.V_SIGGEN_FREQ,
        CONST_PSBSMP.V_SIGGEN_AMPLITUDE,
        CONST_PSBSMP.V_SIGGEN_OFFSET,
        CONST_PSBSMP.V_SIGGEN_AUX_PARAM,
        CONST_PSBSMP.V_WFMREF_SELECTED,
        CONST_PSBSMP.V_WFMREF_SYNC_MODE,
        CONST_PSBSMP.V_WFMREF_FREQUENCY,
        CONST_PSBSMP.V_WFMREF_GAIN,
        CONST_PSBSMP.V_WFMREF_OFFSET,
        CONST_PSBSMP.V_WFMREF0_START,
        CONST_PSBSMP.V_WFMREF0_END,
        CONST_PSBSMP.V_WFMREF0_IDX,
        CONST_PSBSMP.V_WFMREF1_START,
        CONST_PSBSMP.V_WFMREF1_END,
        CONST_PSBSMP.V_WFMREF1_IDX,
        CONST_PSBSMP.V_SCOPE_FREQUENCY,
        CONST_PSBSMP.V_SCOPE_DURATION,
        CONST_PSBSMP.V_SCOPE_SRC_DATA,
        # --- FAP variables ---
        CONST_PSBSMP.V_PS_SOFT_INTERLOCKS,
        CONST_PSBSMP.V_PS_HARD_INTERLOCKS,
        CONST_PSBSMP.V_I_LOAD_MEAN,
        CONST_PSBSMP.V_I_LOAD1,
        CONST_PSBSMP.V_I_LOAD2,
        CONST_PSBSMP.V_V_DCLINK,
        CONST_PSBSMP.V_I_IGBT_1,
        CONST_PSBSMP.V_I_IGBT_2,
        CONST_PSBSMP.V_DUTY_CYCLE_1,
        CONST_PSBSMP.V_DUTY_CYCLE_2,
        CONST_PSBSMP.V_DUTY_DIFF,
        CONST_PSBSMP.V_V_INPUT_IIB,
        CONST_PSBSMP.V_V_OUTPUT_IIB,
        CONST_PSBSMP.V_I_IGBT_1_IIB,
        CONST_PSBSMP.V_I_IGBT_2_IIB,
        CONST_PSBSMP.V_TEMP_IGBT_1_IIB,
        CONST_PSBSMP.V_TEMP_IGBT_2_IIB,
        CONST_PSBSMP.V_V_DRIVER_IIB,
        CONST_PSBSMP.V_I_DRIVER_1_IIB,
        CONST_PSBSMP.V_I_DRIVER_2_IIB,
        CONST_PSBSMP.V_TEMP_INDUCTOR_IIB,
        CONST_PSBSMP.V_TEMP_HEATSINK_IIB,
        CONST_PSBSMP.V_I_LEAKAGE_IIB,
        CONST_PSBSMP.V_TEMP_BOARD_IIB,
        CONST_PSBSMP.V_RH_IIB,
        CONST_PSBSMP.V_IIB_INTERLOCKS,
        CONST_PSBSMP.V_IIB_ALARMS,)
    groups[_const.G_SCAN] = (
        # --- common variables
        CONST_PSBSMP.V_PS_STATUS,
        CONST_PSBSMP.V_PS_SETPOINT,
        CONST_PSBSMP.V_PS_REFERENCE,
        # CONST_PSBSMP.V_FIRMWARE_VERSION,
        CONST_PSBSMP.V_COUNTER_SET_SLOWREF,
        CONST_PSBSMP.V_COUNTER_SYNC_PULSE,
        CONST_PSBSMP.V_SIGGEN_ENABLE,
        CONST_PSBSMP.V_SIGGEN_TYPE,
        CONST_PSBSMP.V_SIGGEN_NUM_CYCLES,
        CONST_PSBSMP.V_SIGGEN_N,
        CONST_PSBSMP.V_SIGGEN_FREQ,
        CONST_PSBSMP.V_SIGGEN_AMPLITUDE,
        CONST_PSBSMP.V_SIGGEN_OFFSET,
        CONST_PSBSMP.V_SIGGEN_AUX_PARAM,
        CONST_PSBSMP.V_WFMREF_SELECTED,
        CONST_PSBSMP.V_WFMREF_SYNC_MODE,
        CONST_PSBSMP.V_WFMREF_FREQUENCY,
        CONST_PSBSMP.V_WFMREF_GAIN,
        CONST_PSBSMP.V_WFMREF_OFFSET,
        CONST_PSBSMP.V_WFMREF0_START,
        CONST_PSBSMP.V_WFMREF0_END,
        CONST_PSBSMP.V_WFMREF0_IDX,
        CONST_PSBSMP.V_WFMREF1_START,
        CONST_PSBSMP.V_WFMREF1_END,
        CONST_PSBSMP.V_WFMREF1_IDX,
        CONST_PSBSMP.V_SCOPE_FREQUENCY,
        CONST_PSBSMP.V_SCOPE_DURATION,
        CONST_PSBSMP.V_SCOPE_SRC_DATA,
        # --- FAP variables ---
        CONST_PSBSMP.V_PS_SOFT_INTERLOCKS,
        CONST_PSBSMP.V_PS_HARD_INTERLOCKS,
        CONST_PSBSMP.V_I_LOAD_MEAN,
        CONST_PSBSMP.V_I_LOAD1,
        CONST_PSBSMP.V_I_LOAD2,
        CONST_PSBSMP.V_V_DCLINK,
        CONST_PSBSMP.V_I_IGBT_1,
        CONST_PSBSMP.V_I_IGBT_2,
        CONST_PSBSMP.V_DUTY_CYCLE_1,
        CONST_PSBSMP.V_DUTY_CYCLE_2,
        CONST_PSBSMP.V_DUTY_DIFF,
        CONST_PSBSMP.V_V_INPUT_IIB,
        CONST_PSBSMP.V_V_OUTPUT_IIB,
        CONST_PSBSMP.V_I_IGBT_1_IIB,
        CONST_PSBSMP.V_I_IGBT_2_IIB,
        CONST_PSBSMP.V_TEMP_IGBT_1_IIB,
        CONST_PSBSMP.V_TEMP_IGBT_2_IIB,
        CONST_PSBSMP.V_V_DRIVER_IIB,
        CONST_PSBSMP.V_I_DRIVER_1_IIB,
        CONST_PSBSMP.V_I_DRIVER_2_IIB,
        CONST_PSBSMP.V_TEMP_INDUCTOR_IIB,
        CONST_PSBSMP.V_TEMP_HEATSINK_IIB,
        CONST_PSBSMP.V_I_LEAKAGE_IIB,
        CONST_PSBSMP.V_TEMP_BOARD_IIB,
        CONST_PSBSMP.V_RH_IIB,
        CONST_PSBSMP.V_IIB_INTERLOCKS,
        CONST_PSBSMP.V_IIB_ALARMS,)


class PRUCParmsFAP_4P:
    """FAC-specific PRUC parameters.

    Represent FAP_4P
    """

    FREQ_SCAN = 5.0  # [Hz]

    # PS model parms
    model = _PSModelFactory.create('FAP_4P')
    CONST_PSBSMP = model.bsmp_constants
    Entities = model.entities

    groups = dict()
    # reserved variable groups (not to be used)
    groups[_const.G_ALL] = tuple(sorted(Entities.list_variables(0)))
    groups[_const.G_READONLY] = tuple(sorted(Entities.list_variables(1)))
    groups[_const.G_WRITE] = tuple(sorted(Entities.list_variables(2)))
    # new variable groups useful for PRUController.
    groups[_const.G_ALLRELEVANT] = (
        # --- common variables
        CONST_PSBSMP.V_PS_STATUS,
        CONST_PSBSMP.V_PS_SETPOINT,
        CONST_PSBSMP.V_PS_REFERENCE,
        # CONST_PSBSMP.V_FIRMWARE_VERSION,
        CONST_PSBSMP.V_COUNTER_SET_SLOWREF,
        CONST_PSBSMP.V_COUNTER_SYNC_PULSE,
        CONST_PSBSMP.V_SIGGEN_ENABLE,
        CONST_PSBSMP.V_SIGGEN_TYPE,
        CONST_PSBSMP.V_SIGGEN_NUM_CYCLES,
        CONST_PSBSMP.V_SIGGEN_N,
        CONST_PSBSMP.V_SIGGEN_FREQ,
        CONST_PSBSMP.V_SIGGEN_AMPLITUDE,
        CONST_PSBSMP.V_SIGGEN_OFFSET,
        CONST_PSBSMP.V_SIGGEN_AUX_PARAM,
        CONST_PSBSMP.V_WFMREF_SELECTED,
        CONST_PSBSMP.V_WFMREF_SYNC_MODE,
        CONST_PSBSMP.V_WFMREF_FREQUENCY,
        CONST_PSBSMP.V_WFMREF_GAIN,
        CONST_PSBSMP.V_WFMREF_OFFSET,
        CONST_PSBSMP.V_WFMREF0_START,
        CONST_PSBSMP.V_WFMREF0_END,
        CONST_PSBSMP.V_WFMREF0_IDX,
        CONST_PSBSMP.V_WFMREF1_START,
        CONST_PSBSMP.V_WFMREF1_END,
        CONST_PSBSMP.V_WFMREF1_IDX,
        CONST_PSBSMP.V_SCOPE_FREQUENCY,
        CONST_PSBSMP.V_SCOPE_DURATION,
        CONST_PSBSMP.V_SCOPE_SRC_DATA,
        # --- FAP_4P variables ---
        CONST_PSBSMP.V_PS_SOFT_INTERLOCKS,
        CONST_PSBSMP.V_PS_HARD_INTERLOCKS,
        CONST_PSBSMP.V_I_LOAD1,
        CONST_PSBSMP.V_I_LOAD2,
        CONST_PSBSMP.V_I_LOAD_MEAN,
        CONST_PSBSMP.V_V_LOAD,
        CONST_PSBSMP.V_I_IGBT_1_1,
        CONST_PSBSMP.V_I_IGBT_2_1,
        CONST_PSBSMP.V_I_IGBT_1_2,
        CONST_PSBSMP.V_I_IGBT_2_2,
        CONST_PSBSMP.V_I_IGBT_1_3,
        CONST_PSBSMP.V_I_IGBT_2_3,
        CONST_PSBSMP.V_I_IGBT_1_4,
        CONST_PSBSMP.V_I_IGBT_2_4,
        CONST_PSBSMP.V_V_DCLINK_1,
        CONST_PSBSMP.V_V_DCLINK_2,
        CONST_PSBSMP.V_V_DCLINK_3,
        CONST_PSBSMP.V_V_DCLINK_4,
        CONST_PSBSMP.V_DUTY_MEAN,
        CONST_PSBSMP.V_DUTY_CYCLE_1_1,
        CONST_PSBSMP.V_DUTY_CYCLE_2_1,
        CONST_PSBSMP.V_DUTY_CYCLE_1_2,
        CONST_PSBSMP.V_DUTY_CYCLE_2_2,
        CONST_PSBSMP.V_DUTY_CYCLE_1_3,
        CONST_PSBSMP.V_DUTY_CYCLE_2_3,
        CONST_PSBSMP.V_DUTY_CYCLE_1_4,
        CONST_PSBSMP.V_DUTY_CYCLE_2_4,
        CONST_PSBSMP.V_V_INPUT_IIB_1,
        CONST_PSBSMP.V_V_OUTPUT_IIB_1,
        CONST_PSBSMP.V_I_IGBT_1_IIB_1,
        CONST_PSBSMP.V_I_IGBT_2_IIB_1,
        CONST_PSBSMP.V_TEMP_IGBT_1_IIB_1,
        CONST_PSBSMP.V_TEMP_IGBT_2_IIB_1,
        CONST_PSBSMP.V_V_DRIVER_IIB_1,
        CONST_PSBSMP.V_I_DRIVER_1_IIB_1,
        CONST_PSBSMP.V_I_DRIVER_2_IIB_1,
        CONST_PSBSMP.V_TEMP_INDUCTOR_IIB_1,
        CONST_PSBSMP.V_TEMP_HEATSINK_IIB_1,
        CONST_PSBSMP.V_I_LEAKAGE_IIB_1,
        CONST_PSBSMP.V_TEMP_BOARD_IIB_1,
        CONST_PSBSMP.V_RH_IIB_1,
        CONST_PSBSMP.V_IIB_INTERLOCKS_1,
        CONST_PSBSMP.V_IIB_ALARMS_1,
        CONST_PSBSMP.V_V_INPUT_IIB_2,
        CONST_PSBSMP.V_V_OUTPUT_IIB_2,
        CONST_PSBSMP.V_I_IGBT_1_IIB_2,
        CONST_PSBSMP.V_I_IGBT_2_IIB_2,
        CONST_PSBSMP.V_TEMP_IGBT_1_IIB_2,
        CONST_PSBSMP.V_TEMP_IGBT_2_IIB_2,
        CONST_PSBSMP.V_V_DRIVER_IIB_2,
        CONST_PSBSMP.V_I_DRIVER_1_IIB_2,
        CONST_PSBSMP.V_I_DRIVER_2_IIB_2,
        CONST_PSBSMP.V_TEMP_INDUCTOR_IIB_2,
        CONST_PSBSMP.V_TEMP_HEATSINK_IIB_2,
        CONST_PSBSMP.V_I_LEAKAGE_IIB_2,
        CONST_PSBSMP.V_TEMP_BOARD_IIB_2,
        CONST_PSBSMP.V_RH_IIB_2,
        CONST_PSBSMP.V_IIB_INTERLOCKS_2,
        CONST_PSBSMP.V_IIB_ALARMS_2,
        CONST_PSBSMP.V_V_INPUT_IIB_3,
        CONST_PSBSMP.V_V_OUTPUT_IIB_3,
        CONST_PSBSMP.V_I_IGBT_1_IIB_3,
        CONST_PSBSMP.V_I_IGBT_2_IIB_3,
        CONST_PSBSMP.V_TEMP_IGBT_1_IIB_3,
        CONST_PSBSMP.V_TEMP_IGBT_2_IIB_3,
        CONST_PSBSMP.V_V_DRIVER_IIB_3,
        CONST_PSBSMP.V_I_DRIVER_1_IIB_3,
        CONST_PSBSMP.V_I_DRIVER_2_IIB_3,
        CONST_PSBSMP.V_TEMP_INDUCTOR_IIB_3,
        CONST_PSBSMP.V_TEMP_HEATSINK_IIB_3,
        CONST_PSBSMP.V_I_LEAKAGE_IIB_3,
        CONST_PSBSMP.V_TEMP_BOARD_IIB_3,
        CONST_PSBSMP.V_RH_IIB_3,
        CONST_PSBSMP.V_IIB_INTERLOCKS_3,
        CONST_PSBSMP.V_IIB_ALARMS_3,
        CONST_PSBSMP.V_V_INPUT_IIB_4,
        CONST_PSBSMP.V_V_OUTPUT_IIB_4,
        CONST_PSBSMP.V_I_IGBT_1_IIB_4,
        CONST_PSBSMP.V_I_IGBT_2_IIB_4,
        CONST_PSBSMP.V_TEMP_IGBT_1_IIB_4,
        CONST_PSBSMP.V_TEMP_IGBT_2_IIB_4,
        CONST_PSBSMP.V_V_DRIVER_IIB_4,
        CONST_PSBSMP.V_I_DRIVER_1_IIB_4,
        CONST_PSBSMP.V_I_DRIVER_2_IIB_4,
        CONST_PSBSMP.V_TEMP_INDUCTOR_IIB_4,
        CONST_PSBSMP.V_TEMP_HEATSINK_IIB_4,
        CONST_PSBSMP.V_I_LEAKAGE_IIB_4,
        CONST_PSBSMP.V_TEMP_BOARD_IIB_4,
        CONST_PSBSMP.V_RH_IIB_4,
        CONST_PSBSMP.V_IIB_INTERLOCKS_4,
        CONST_PSBSMP.V_IIB_ALARMS_4,)
    groups[_const.G_SCAN] = (
        # --- common variables
        CONST_PSBSMP.V_PS_STATUS,
        CONST_PSBSMP.V_PS_SETPOINT,
        CONST_PSBSMP.V_PS_REFERENCE,
        # CONST_PSBSMP.V_FIRMWARE_VERSION,
        CONST_PSBSMP.V_COUNTER_SET_SLOWREF,
        CONST_PSBSMP.V_COUNTER_SYNC_PULSE,
        CONST_PSBSMP.V_SIGGEN_ENABLE,
        CONST_PSBSMP.V_SIGGEN_TYPE,
        CONST_PSBSMP.V_SIGGEN_NUM_CYCLES,
        CONST_PSBSMP.V_SIGGEN_N,
        CONST_PSBSMP.V_SIGGEN_FREQ,
        CONST_PSBSMP.V_SIGGEN_AMPLITUDE,
        CONST_PSBSMP.V_SIGGEN_OFFSET,
        CONST_PSBSMP.V_SIGGEN_AUX_PARAM,
        CONST_PSBSMP.V_WFMREF_SELECTED,
        CONST_PSBSMP.V_WFMREF_SYNC_MODE,
        CONST_PSBSMP.V_WFMREF_FREQUENCY,
        CONST_PSBSMP.V_WFMREF_GAIN,
        CONST_PSBSMP.V_WFMREF_OFFSET,
        CONST_PSBSMP.V_WFMREF0_START,
        CONST_PSBSMP.V_WFMREF0_END,
        CONST_PSBSMP.V_WFMREF0_IDX,
        CONST_PSBSMP.V_WFMREF1_START,
        CONST_PSBSMP.V_WFMREF1_END,
        CONST_PSBSMP.V_WFMREF1_IDX,
        CONST_PSBSMP.V_SCOPE_FREQUENCY,
        CONST_PSBSMP.V_SCOPE_DURATION,
        CONST_PSBSMP.V_SCOPE_SRC_DATA,
        # --- FAP_4P variables ---
        CONST_PSBSMP.V_PS_SOFT_INTERLOCKS,
        CONST_PSBSMP.V_PS_HARD_INTERLOCKS,
        CONST_PSBSMP.V_I_LOAD1,
        CONST_PSBSMP.V_I_LOAD2,
        CONST_PSBSMP.V_I_LOAD_MEAN,
        CONST_PSBSMP.V_V_LOAD,
        CONST_PSBSMP.V_I_IGBT_1_1,
        CONST_PSBSMP.V_I_IGBT_2_1,
        CONST_PSBSMP.V_I_IGBT_1_2,
        CONST_PSBSMP.V_I_IGBT_2_2,
        CONST_PSBSMP.V_I_IGBT_1_3,
        CONST_PSBSMP.V_I_IGBT_2_3,
        CONST_PSBSMP.V_I_IGBT_1_4,
        CONST_PSBSMP.V_I_IGBT_2_4,
        CONST_PSBSMP.V_V_DCLINK_1,
        CONST_PSBSMP.V_V_DCLINK_2,
        CONST_PSBSMP.V_V_DCLINK_3,
        CONST_PSBSMP.V_V_DCLINK_4,
        CONST_PSBSMP.V_DUTY_MEAN,
        CONST_PSBSMP.V_DUTY_CYCLE_1_1,
        CONST_PSBSMP.V_DUTY_CYCLE_2_1,
        CONST_PSBSMP.V_DUTY_CYCLE_1_2,
        CONST_PSBSMP.V_DUTY_CYCLE_2_2,
        CONST_PSBSMP.V_DUTY_CYCLE_1_3,
        CONST_PSBSMP.V_DUTY_CYCLE_2_3,
        CONST_PSBSMP.V_DUTY_CYCLE_1_4,
        CONST_PSBSMP.V_DUTY_CYCLE_2_4,
        CONST_PSBSMP.V_V_INPUT_IIB_1,
        CONST_PSBSMP.V_V_OUTPUT_IIB_1,
        CONST_PSBSMP.V_I_IGBT_1_IIB_1,
        CONST_PSBSMP.V_I_IGBT_2_IIB_1,
        CONST_PSBSMP.V_TEMP_IGBT_1_IIB_1,
        CONST_PSBSMP.V_TEMP_IGBT_2_IIB_1,
        CONST_PSBSMP.V_V_DRIVER_IIB_1,
        CONST_PSBSMP.V_I_DRIVER_1_IIB_1,
        CONST_PSBSMP.V_I_DRIVER_2_IIB_1,
        CONST_PSBSMP.V_TEMP_INDUCTOR_IIB_1,
        CONST_PSBSMP.V_TEMP_HEATSINK_IIB_1,
        CONST_PSBSMP.V_I_LEAKAGE_IIB_1,
        CONST_PSBSMP.V_TEMP_BOARD_IIB_1,
        CONST_PSBSMP.V_RH_IIB_1,
        CONST_PSBSMP.V_IIB_INTERLOCKS_1,
        CONST_PSBSMP.V_IIB_ALARMS_1,
        CONST_PSBSMP.V_V_INPUT_IIB_2,
        CONST_PSBSMP.V_V_OUTPUT_IIB_2,
        CONST_PSBSMP.V_I_IGBT_1_IIB_2,
        CONST_PSBSMP.V_I_IGBT_2_IIB_2,
        CONST_PSBSMP.V_TEMP_IGBT_1_IIB_2,
        CONST_PSBSMP.V_TEMP_IGBT_2_IIB_2,
        CONST_PSBSMP.V_V_DRIVER_IIB_2,
        CONST_PSBSMP.V_I_DRIVER_1_IIB_2,
        CONST_PSBSMP.V_I_DRIVER_2_IIB_2,
        CONST_PSBSMP.V_TEMP_INDUCTOR_IIB_2,
        CONST_PSBSMP.V_TEMP_HEATSINK_IIB_2,
        CONST_PSBSMP.V_I_LEAKAGE_IIB_2,
        CONST_PSBSMP.V_TEMP_BOARD_IIB_2,
        CONST_PSBSMP.V_RH_IIB_2,
        CONST_PSBSMP.V_IIB_INTERLOCKS_2,
        CONST_PSBSMP.V_IIB_ALARMS_2,
        CONST_PSBSMP.V_V_INPUT_IIB_3,
        CONST_PSBSMP.V_V_OUTPUT_IIB_3,
        CONST_PSBSMP.V_I_IGBT_1_IIB_3,
        CONST_PSBSMP.V_I_IGBT_2_IIB_3,
        CONST_PSBSMP.V_TEMP_IGBT_1_IIB_3,
        CONST_PSBSMP.V_TEMP_IGBT_2_IIB_3,
        CONST_PSBSMP.V_V_DRIVER_IIB_3,
        CONST_PSBSMP.V_I_DRIVER_1_IIB_3,
        CONST_PSBSMP.V_I_DRIVER_2_IIB_3,
        CONST_PSBSMP.V_TEMP_INDUCTOR_IIB_3,
        CONST_PSBSMP.V_TEMP_HEATSINK_IIB_3,
        CONST_PSBSMP.V_I_LEAKAGE_IIB_3,
        CONST_PSBSMP.V_TEMP_BOARD_IIB_3,
        CONST_PSBSMP.V_RH_IIB_3,
        CONST_PSBSMP.V_IIB_INTERLOCKS_3,
        CONST_PSBSMP.V_IIB_ALARMS_3,
        CONST_PSBSMP.V_V_INPUT_IIB_4,
        CONST_PSBSMP.V_V_OUTPUT_IIB_4,
        CONST_PSBSMP.V_I_IGBT_1_IIB_4,
        CONST_PSBSMP.V_I_IGBT_2_IIB_4,
        CONST_PSBSMP.V_TEMP_IGBT_1_IIB_4,
        CONST_PSBSMP.V_TEMP_IGBT_2_IIB_4,
        CONST_PSBSMP.V_V_DRIVER_IIB_4,
        CONST_PSBSMP.V_I_DRIVER_1_IIB_4,
        CONST_PSBSMP.V_I_DRIVER_2_IIB_4,
        CONST_PSBSMP.V_TEMP_INDUCTOR_IIB_4,
        CONST_PSBSMP.V_TEMP_HEATSINK_IIB_4,
        CONST_PSBSMP.V_I_LEAKAGE_IIB_4,
        CONST_PSBSMP.V_TEMP_BOARD_IIB_4,
        CONST_PSBSMP.V_RH_IIB_4,
        CONST_PSBSMP.V_IIB_INTERLOCKS_4,
        CONST_PSBSMP.V_IIB_ALARMS_4,)


class PRUCParmsFAP_2P2S:
    """FAC_2P2S-specific PRUC parameters.

    Represent FAP_2P2S
    """

    FREQ_SCAN = 5.0  # [Hz]

    # PS model parms
    model = _PSModelFactory.create('FAP_2P2S')
    CONST_PSBSMP = model.bsmp_constants
    Entities = model.entities

    groups = dict()
    # reserved variable groups (not to be used)
    groups[_const.G_ALL] = tuple(sorted(Entities.list_variables(0)))
    groups[_const.G_READONLY] = tuple(sorted(Entities.list_variables(1)))
    groups[_const.G_WRITE] = tuple(sorted(Entities.list_variables(2)))
    # new variable groups usefull for PRUController.
    groups[_const.G_ALLRELEVANT] = (
        # --- common variables
        CONST_PSBSMP.V_PS_STATUS,
        CONST_PSBSMP.V_PS_SETPOINT,
        CONST_PSBSMP.V_PS_REFERENCE,
        CONST_PSBSMP.V_FIRMWARE_VERSION,
        CONST_PSBSMP.V_COUNTER_SET_SLOWREF,
        CONST_PSBSMP.V_COUNTER_SYNC_PULSE,
        CONST_PSBSMP.V_SIGGEN_ENABLE,
        CONST_PSBSMP.V_SIGGEN_TYPE,
        CONST_PSBSMP.V_SIGGEN_NUM_CYCLES,
        CONST_PSBSMP.V_SIGGEN_N,
        CONST_PSBSMP.V_SIGGEN_FREQ,
        CONST_PSBSMP.V_SIGGEN_AMPLITUDE,
        CONST_PSBSMP.V_SIGGEN_OFFSET,
        CONST_PSBSMP.V_SIGGEN_AUX_PARAM,
        CONST_PSBSMP.V_WFMREF_SELECTED,
        CONST_PSBSMP.V_WFMREF_SYNC_MODE,
        CONST_PSBSMP.V_WFMREF_FREQUENCY,
        CONST_PSBSMP.V_WFMREF_GAIN,
        CONST_PSBSMP.V_WFMREF_OFFSET,
        CONST_PSBSMP.V_WFMREF0_START,
        CONST_PSBSMP.V_WFMREF0_END,
        CONST_PSBSMP.V_WFMREF0_IDX,
        CONST_PSBSMP.V_WFMREF1_START,
        CONST_PSBSMP.V_WFMREF1_END,
        CONST_PSBSMP.V_WFMREF1_IDX,
        CONST_PSBSMP.V_SCOPE_FREQUENCY,
        CONST_PSBSMP.V_SCOPE_DURATION,
        CONST_PSBSMP.V_SCOPE_SRC_DATA,
        # --- FAP_2P2S variables ---
        CONST_PSBSMP.V_PS_SOFT_INTERLOCKS,
        CONST_PSBSMP.V_PS_HARD_INTERLOCKS,
        CONST_PSBSMP.V_I_LOAD_MEAN,
        CONST_PSBSMP.V_I_LOAD1,
        CONST_PSBSMP.V_I_LOAD2,
        CONST_PSBSMP.V_I_ARM_1,
        CONST_PSBSMP.V_I_ARM_2,
        CONST_PSBSMP.V_I_IGBT_1_1,
        CONST_PSBSMP.V_I_IGBT_2_1,
        CONST_PSBSMP.V_I_IGBT_1_2,
        CONST_PSBSMP.V_I_IGBT_2_2,
        CONST_PSBSMP.V_I_IGBT_1_3,
        CONST_PSBSMP.V_I_IGBT_2_3,
        CONST_PSBSMP.V_I_IGBT_1_4,
        CONST_PSBSMP.V_I_IGBT_2_4,
        CONST_PSBSMP.V_I_MOD_1,
        CONST_PSBSMP.V_I_MOD_2,
        CONST_PSBSMP.V_I_MOD_3,
        CONST_PSBSMP.V_I_MOD_4,
        CONST_PSBSMP.V_V_DCLINK_1,
        CONST_PSBSMP.V_V_DCLINK_2,
        CONST_PSBSMP.V_V_DCLINK_3,
        CONST_PSBSMP.V_V_DCLINK_4,
        CONST_PSBSMP.V_DUTY_MEAN,
        CONST_PSBSMP.V_DUTY_ARMS_DIFF,
        CONST_PSBSMP.V_DUTY_CYCLE_1_1,
        CONST_PSBSMP.V_DUTY_CYCLE_2_1,
        CONST_PSBSMP.V_DUTY_CYCLE_1_2,
        CONST_PSBSMP.V_DUTY_CYCLE_2_2,
        CONST_PSBSMP.V_DUTY_CYCLE_1_3,
        CONST_PSBSMP.V_DUTY_CYCLE_2_3,
        CONST_PSBSMP.V_DUTY_CYCLE_1_4,
        CONST_PSBSMP.V_DUTY_CYCLE_2_4,
        CONST_PSBSMP.V_V_INPUT_IIB_1,
        CONST_PSBSMP.V_V_OUTPUT_IIB_1,
        CONST_PSBSMP.V_I_IGBT_1_IIB_1,
        CONST_PSBSMP.V_I_IGBT_2_IIB_1,
        CONST_PSBSMP.V_TEMP_IGBT_1_IIB_1,
        CONST_PSBSMP.V_TEMP_IGBT_2_IIB_1,
        CONST_PSBSMP.V_V_DRIVER_IIB_1,
        CONST_PSBSMP.V_I_DRIVER_1_IIB_1,
        CONST_PSBSMP.V_I_DRIVER_2_IIB_1,
        CONST_PSBSMP.V_TEMP_INDUCTOR_IIB_1,
        CONST_PSBSMP.V_TEMP_HEATSINK_IIB_1,
        CONST_PSBSMP.V_I_LEAKAGE_IIB_1,
        CONST_PSBSMP.V_TEMP_BOARD_IIB_1,
        CONST_PSBSMP.V_RH_IIB_1,
        CONST_PSBSMP.V_IIB_INTERLOCKS_1,
        CONST_PSBSMP.V_IIB_ALARMS_1,
        CONST_PSBSMP.V_V_INPUT_IIB_2,
        CONST_PSBSMP.V_V_OUTPUT_IIB_2,
        CONST_PSBSMP.V_I_IGBT_1_IIB_2,
        CONST_PSBSMP.V_I_IGBT_2_IIB_2,
        CONST_PSBSMP.V_TEMP_IGBT_1_IIB_2,
        CONST_PSBSMP.V_TEMP_IGBT_2_IIB_2,
        CONST_PSBSMP.V_V_DRIVER_IIB_2,
        CONST_PSBSMP.V_I_DRIVER_1_IIB_2,
        CONST_PSBSMP.V_I_DRIVER_2_IIB_2,
        CONST_PSBSMP.V_TEMP_INDUCTOR_IIB_2,
        CONST_PSBSMP.V_TEMP_HEATSINK_IIB_2,
        CONST_PSBSMP.V_I_LEAKAGE_IIB_2,
        CONST_PSBSMP.V_TEMP_BOARD_IIB_2,
        CONST_PSBSMP.V_RH_IIB_2,
        CONST_PSBSMP.V_IIB_INTERLOCKS_2,
        CONST_PSBSMP.V_IIB_ALARMS_2,
        CONST_PSBSMP.V_V_INPUT_IIB_3,
        CONST_PSBSMP.V_V_OUTPUT_IIB_3,
        CONST_PSBSMP.V_I_IGBT_1_IIB_3,
        CONST_PSBSMP.V_I_IGBT_2_IIB_3,
        CONST_PSBSMP.V_TEMP_IGBT_1_IIB_3,
        CONST_PSBSMP.V_TEMP_IGBT_2_IIB_3,
        CONST_PSBSMP.V_V_DRIVER_IIB_3,
        CONST_PSBSMP.V_I_DRIVER_1_IIB_3,
        CONST_PSBSMP.V_I_DRIVER_2_IIB_3,
        CONST_PSBSMP.V_TEMP_INDUCTOR_IIB_3,
        CONST_PSBSMP.V_TEMP_HEATSINK_IIB_3,
        CONST_PSBSMP.V_I_LEAKAGE_IIB_3,
        CONST_PSBSMP.V_TEMP_BOARD_IIB_3,
        CONST_PSBSMP.V_RH_IIB_3,
        CONST_PSBSMP.V_IIB_INTERLOCKS_3,
        CONST_PSBSMP.V_IIB_ALARMS_3,
        CONST_PSBSMP.V_V_INPUT_IIB_4,
        CONST_PSBSMP.V_V_OUTPUT_IIB_4,
        CONST_PSBSMP.V_I_IGBT_1_IIB_4,
        CONST_PSBSMP.V_I_IGBT_2_IIB_4,
        CONST_PSBSMP.V_TEMP_IGBT_1_IIB_4,
        CONST_PSBSMP.V_TEMP_IGBT_2_IIB_4,
        CONST_PSBSMP.V_V_DRIVER_IIB_4,
        CONST_PSBSMP.V_I_DRIVER_1_IIB_4,
        CONST_PSBSMP.V_I_DRIVER_2_IIB_4,
        CONST_PSBSMP.V_TEMP_INDUCTOR_IIB_4,
        CONST_PSBSMP.V_TEMP_HEATSINK_IIB_4,
        CONST_PSBSMP.V_I_LEAKAGE_IIB_4,
        CONST_PSBSMP.V_TEMP_BOARD_IIB_4,
        CONST_PSBSMP.V_RH_IIB_4,
        CONST_PSBSMP.V_IIB_INTERLOCKS_4,
        CONST_PSBSMP.V_IIB_ALARMS_4)

    groups[_const.G_SCAN] = (
        # --- common variables
        # --- common variables
        CONST_PSBSMP.V_PS_STATUS,
        CONST_PSBSMP.V_PS_SETPOINT,
        CONST_PSBSMP.V_PS_REFERENCE,
        # CONST_PSBSMP.V_FIRMWARE_VERSION,
        CONST_PSBSMP.V_COUNTER_SET_SLOWREF,
        CONST_PSBSMP.V_COUNTER_SYNC_PULSE,
        CONST_PSBSMP.V_SIGGEN_ENABLE,
        CONST_PSBSMP.V_SIGGEN_TYPE,
        CONST_PSBSMP.V_SIGGEN_NUM_CYCLES,
        CONST_PSBSMP.V_SIGGEN_N,
        CONST_PSBSMP.V_SIGGEN_FREQ,
        CONST_PSBSMP.V_SIGGEN_AMPLITUDE,
        CONST_PSBSMP.V_SIGGEN_OFFSET,
        CONST_PSBSMP.V_SIGGEN_AUX_PARAM,
        CONST_PSBSMP.V_WFMREF_SELECTED,
        CONST_PSBSMP.V_WFMREF_SYNC_MODE,
        CONST_PSBSMP.V_WFMREF_FREQUENCY,
        CONST_PSBSMP.V_WFMREF_GAIN,
        CONST_PSBSMP.V_WFMREF_OFFSET,
        CONST_PSBSMP.V_WFMREF0_START,
        CONST_PSBSMP.V_WFMREF0_END,
        CONST_PSBSMP.V_WFMREF0_IDX,
        CONST_PSBSMP.V_WFMREF1_START,
        CONST_PSBSMP.V_WFMREF1_END,
        CONST_PSBSMP.V_WFMREF1_IDX,
        CONST_PSBSMP.V_SCOPE_FREQUENCY,
        CONST_PSBSMP.V_SCOPE_DURATION,
        CONST_PSBSMP.V_SCOPE_SRC_DATA,
        # --- FAP_2P2S variables ---
        CONST_PSBSMP.V_PS_SOFT_INTERLOCKS,
        CONST_PSBSMP.V_PS_HARD_INTERLOCKS,
        CONST_PSBSMP.V_I_LOAD_MEAN,
        CONST_PSBSMP.V_I_LOAD1,
        CONST_PSBSMP.V_I_LOAD2,
        CONST_PSBSMP.V_I_ARM_1,
        CONST_PSBSMP.V_I_ARM_2,
        CONST_PSBSMP.V_I_IGBT_1_1,
        CONST_PSBSMP.V_I_IGBT_2_1,
        CONST_PSBSMP.V_I_IGBT_1_2,
        CONST_PSBSMP.V_I_IGBT_2_2,
        CONST_PSBSMP.V_I_IGBT_1_3,
        CONST_PSBSMP.V_I_IGBT_2_3,
        CONST_PSBSMP.V_I_IGBT_1_4,
        CONST_PSBSMP.V_I_IGBT_2_4,
        CONST_PSBSMP.V_I_MOD_1,
        CONST_PSBSMP.V_I_MOD_2,
        CONST_PSBSMP.V_I_MOD_3,
        CONST_PSBSMP.V_I_MOD_4,
        CONST_PSBSMP.V_V_DCLINK_1,
        CONST_PSBSMP.V_V_DCLINK_2,
        CONST_PSBSMP.V_V_DCLINK_3,
        CONST_PSBSMP.V_V_DCLINK_4,
        CONST_PSBSMP.V_DUTY_MEAN,
        CONST_PSBSMP.V_DUTY_ARMS_DIFF,
        CONST_PSBSMP.V_DUTY_CYCLE_1_1,
        CONST_PSBSMP.V_DUTY_CYCLE_2_1,
        CONST_PSBSMP.V_DUTY_CYCLE_1_2,
        CONST_PSBSMP.V_DUTY_CYCLE_2_2,
        CONST_PSBSMP.V_DUTY_CYCLE_1_3,
        CONST_PSBSMP.V_DUTY_CYCLE_2_3,
        CONST_PSBSMP.V_DUTY_CYCLE_1_4,
        CONST_PSBSMP.V_DUTY_CYCLE_2_4,
        CONST_PSBSMP.V_V_INPUT_IIB_1,
        CONST_PSBSMP.V_V_OUTPUT_IIB_1,
        CONST_PSBSMP.V_I_IGBT_1_IIB_1,
        CONST_PSBSMP.V_I_IGBT_2_IIB_1,
        CONST_PSBSMP.V_TEMP_IGBT_1_IIB_1,
        CONST_PSBSMP.V_TEMP_IGBT_2_IIB_1,
        CONST_PSBSMP.V_V_DRIVER_IIB_1,
        CONST_PSBSMP.V_I_DRIVER_1_IIB_1,
        CONST_PSBSMP.V_I_DRIVER_2_IIB_1,
        CONST_PSBSMP.V_TEMP_INDUCTOR_IIB_1,
        CONST_PSBSMP.V_TEMP_HEATSINK_IIB_1,
        CONST_PSBSMP.V_I_LEAKAGE_IIB_1,
        CONST_PSBSMP.V_TEMP_BOARD_IIB_1,
        CONST_PSBSMP.V_RH_IIB_1,
        CONST_PSBSMP.V_IIB_INTERLOCKS_1,
        CONST_PSBSMP.V_IIB_ALARMS_1,
        CONST_PSBSMP.V_V_INPUT_IIB_2,
        CONST_PSBSMP.V_V_OUTPUT_IIB_2,
        CONST_PSBSMP.V_I_IGBT_1_IIB_2,
        CONST_PSBSMP.V_I_IGBT_2_IIB_2,
        CONST_PSBSMP.V_TEMP_IGBT_1_IIB_2,
        CONST_PSBSMP.V_TEMP_IGBT_2_IIB_2,
        CONST_PSBSMP.V_V_DRIVER_IIB_2,
        CONST_PSBSMP.V_I_DRIVER_1_IIB_2,
        CONST_PSBSMP.V_I_DRIVER_2_IIB_2,
        CONST_PSBSMP.V_TEMP_INDUCTOR_IIB_2,
        CONST_PSBSMP.V_TEMP_HEATSINK_IIB_2,
        CONST_PSBSMP.V_I_LEAKAGE_IIB_2,
        CONST_PSBSMP.V_TEMP_BOARD_IIB_2,
        CONST_PSBSMP.V_RH_IIB_2,
        CONST_PSBSMP.V_IIB_INTERLOCKS_2,
        CONST_PSBSMP.V_IIB_ALARMS_2,
        CONST_PSBSMP.V_V_INPUT_IIB_3,
        CONST_PSBSMP.V_V_OUTPUT_IIB_3,
        CONST_PSBSMP.V_I_IGBT_1_IIB_3,
        CONST_PSBSMP.V_I_IGBT_2_IIB_3,
        CONST_PSBSMP.V_TEMP_IGBT_1_IIB_3,
        CONST_PSBSMP.V_TEMP_IGBT_2_IIB_3,
        CONST_PSBSMP.V_V_DRIVER_IIB_3,
        CONST_PSBSMP.V_I_DRIVER_1_IIB_3,
        CONST_PSBSMP.V_I_DRIVER_2_IIB_3,
        CONST_PSBSMP.V_TEMP_INDUCTOR_IIB_3,
        CONST_PSBSMP.V_TEMP_HEATSINK_IIB_3,
        CONST_PSBSMP.V_I_LEAKAGE_IIB_3,
        CONST_PSBSMP.V_TEMP_BOARD_IIB_3,
        CONST_PSBSMP.V_RH_IIB_3,
        CONST_PSBSMP.V_IIB_INTERLOCKS_3,
        CONST_PSBSMP.V_IIB_ALARMS_3,
        CONST_PSBSMP.V_V_INPUT_IIB_4,
        CONST_PSBSMP.V_V_OUTPUT_IIB_4,
        CONST_PSBSMP.V_I_IGBT_1_IIB_4,
        CONST_PSBSMP.V_I_IGBT_2_IIB_4,
        CONST_PSBSMP.V_TEMP_IGBT_1_IIB_4,
        CONST_PSBSMP.V_TEMP_IGBT_2_IIB_4,
        CONST_PSBSMP.V_V_DRIVER_IIB_4,
        CONST_PSBSMP.V_I_DRIVER_1_IIB_4,
        CONST_PSBSMP.V_I_DRIVER_2_IIB_4,
        CONST_PSBSMP.V_TEMP_INDUCTOR_IIB_4,
        CONST_PSBSMP.V_TEMP_HEATSINK_IIB_4,
        CONST_PSBSMP.V_I_LEAKAGE_IIB_4,
        CONST_PSBSMP.V_TEMP_BOARD_IIB_4,
        CONST_PSBSMP.V_RH_IIB_4,
        CONST_PSBSMP.V_IIB_INTERLOCKS_4,
        CONST_PSBSMP.V_IIB_ALARMS_4)
